@@ -15,16 +15,21 @@
 #'   to embed the value of the object in the expression (to
 #'   be portable between sessions). See the examples.
 #'
-#'  When you [`tidy()`] this step, a tibble with column `terms` which
-#'  contains the sorting variable(s) or expression(s) is returned. The
-#'  expressions are text representations and are not parsable.
+#'  # Tidying
+#'
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with column
+#'  `terms` which contains the sorting variable(s) or expression(s) is
+#'  returned. The expressions are text representations and are not
+#'  parsable.
+#'
+#' @template case-weights-not-supported
 #'
 #' @family row operation steps
 #' @family dplyr steps
 #' @export
 #' @examples
-#' rec <- recipe( ~ ., data = iris) %>%
-#'   step_arrange(desc(Sepal.Length), 1/Petal.Length)
+#' rec <- recipe(~., data = iris) %>%
+#'   step_arrange(desc(Sepal.Length), 1 / Petal.Length)
 #'
 #' prepped <- prep(rec, training = iris %>% slice(1:75))
 #' tidy(prepped, number = 1)
@@ -35,7 +40,7 @@
 #'   iris %>%
 #'   as_tibble() %>%
 #'   slice(1:75) %>%
-#'   dplyr::arrange(desc(Sepal.Length), 1/Petal.Length)
+#'   dplyr::arrange(desc(Sepal.Length), 1 / Petal.Length)
 #'
 #' rec_train <- bake(prepped, new_data = NULL)
 #' all.equal(dplyr_train, rec_train)
@@ -44,7 +49,7 @@
 #'   iris %>%
 #'   as_tibble() %>%
 #'   slice(76:150) %>%
-#'   dplyr::arrange(desc(Sepal.Length), 1/Petal.Length)
+#'   dplyr::arrange(desc(Sepal.Length), 1 / Petal.Length)
 #' rec_test <- bake(prepped, iris %>% slice(76:150))
 #' all.equal(dplyr_test, rec_test)
 #'
@@ -55,22 +60,18 @@
 #' sort_vars <- c("Sepal.Length", "Petal.Length")
 #'
 #' qq_rec <-
-#'   recipe( ~ ., data = iris) %>%
+#'   recipe(~., data = iris) %>%
 #'   # Embed the `values` object in the call using !!!
 #'   step_arrange(!!!syms(sort_vars)) %>%
 #'   prep(training = iris)
 #'
 #' tidy(qq_rec, number = 1)
-
-step_arrange <- function(
-  recipe, ...,
-  role = NA,
-  trained = FALSE,
-  inputs = NULL,
-  skip = FALSE,
-  id = rand_id("arrange")
-) {
-
+step_arrange <- function(recipe, ...,
+                         role = NA,
+                         trained = FALSE,
+                         inputs = NULL,
+                         skip = FALSE,
+                         id = rand_id("arrange")) {
   inputs <- enquos(...)
 
   add_step(
@@ -119,19 +120,16 @@ bake.step_arrange <- function(object, new_data, ...) {
 
 print.step_arrange <-
   function(x, width = max(20, options()$width - 35), ...) {
-    cat("Row arrangement")
-    if (x$trained) {
-      cat(" [trained]\n")
-    } else {
-      cat("\n")
-    }
+    title <- "Row arrangement using "
+    print_step(x$inputs, x$inputs, x$trained, title, width)
     invisible(x)
   }
 
 #' @rdname tidy.recipe
 #' @export
 tidy.step_arrange <- function(x, ...) {
-  cond_expr <- map(x$inputs, quo_get_expr)
+  cond_expr <- unname(x$inputs)
+  cond_expr <- map(cond_expr, quo_get_expr)
   cond_expr <- map_chr(cond_expr, quo_text, width = options()$width, nlines = 1)
   tibble(
     terms = cond_expr,
