@@ -4,28 +4,19 @@
 #'  that will create new columns that are basis expansions of
 #'  variables using B-splines.
 #'
+#' @inheritParams step_pca
 #' @inheritParams step_center
-#' @param ... One or more selector functions to choose which
-#'  variables are affected by the step. See [selections()]
-#'  for more details. For the `tidy` method, these are not
-#'  currently used.
-#' @param role For model terms created by this step, what analysis
-#'  role should they be assigned?. By default, the function assumes
-#'  that the new columns created from the original variables will be
-#'  used as predictors in a model.
 #' @param objects A list of [splines::bs()] objects
 #'  created once the step has been trained.
-#' @param deg_free The degrees of freedom.
+#' @param deg_free The degrees of freedom for the spline. As the
+#'  degrees of freedom for a spline increase, more flexible and
+#'  complex curves can be generated. When a single degree of freedom is used,
+#'  the result is a rescaled version of the original data.
 #' @param degree Degree of polynomial spline (integer).
 #' @param options A list of options for [splines::bs()]
 #'  which should not include `x`, `degree`, or `df`.
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` which is
-#'  the columns that will be affected and `holiday`.
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept basis_expansion
+#' @template step-return
+#' @family individual transformation steps
 #' @export
 #' @details `step_bs` can create new features from a single variable
 #'  that enable fitting routines to model this variable in a
@@ -34,15 +25,24 @@
 #'  [splines::bs()]. The original variables are removed
 #'  from the data and new columns are added. The naming convention
 #'  for the new variables is `varname_bs_1` and so on.
-#' @examples
-#' library(modeldata)
-#' data(biomass)
 #'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#'  # Tidying
 #'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-#'               data = biomass_tr)
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with column
+#'  `terms` (the columns that will be affected) is returned.
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(biomass, package = "modeldata")
+#'
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
+#'
+#' rec <- recipe(
+#'   HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
+#'   data = biomass_tr
+#' )
 #'
 #' with_splines <- rec %>%
 #'   step_bs(carbon, hydrogen)
@@ -50,9 +50,6 @@
 #'
 #' expanded <- bake(with_splines, biomass_te)
 #' expanded
-#' @seealso [step_poly()] [recipe()] [step_ns()]
-#'   [prep.recipe()] [bake.recipe()]
-
 step_bs <-
   function(recipe,
            ...,
@@ -64,11 +61,10 @@ step_bs <-
            options = list(),
            skip = FALSE,
            id = rand_id("bs")) {
-
     add_step(
       recipe,
       step_bs_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         trained = trained,
         deg_free = deg_free,
         degree = degree,
@@ -97,32 +93,56 @@ step_bs_new <-
     )
   }
 
-bs_wrapper <- function(x, args) {
-  if (!("Boundary.knots" %in% names(args)))
-    args$Boundary.knots <- range(x)
-  args$x <- x
-  bs_obj <- do.call("bs", args)
-  ## don't need to save the original data so keep 1 row
-  out <- matrix(NA, ncol = ncol(bs_obj), nrow = 1)
+bs_statistics <- function(x, args) {
+  # Only do the parameter computations from splines::bs() / splines::ns(), don't evaluate at x.
+  degree <- as.integer(args$degree %||% 3L)
+  intercept <- as.logical(args$intercept %||% FALSE)
+  # This behaves differently from splines::ns() if length(x) is 1
+  boundary <- sort(args$Boundary.knots) %||% range(x)
+
+  # This behaves differently from splines::bs() and splines::ns() if num_knots < 0L
+  # the original implementations issue a warning.
+  if (!is.null(args$df) && is.null(args$knots) && args$df - degree - intercept >= 1L) {
+    num_knots <- args$df - degree - intercept
+    ok <- !is.na(x) & x >= boundary[1L] & x <= boundary[2L]
+    knots <- unname(quantile(x[ok], seq_len(num_knots) / (num_knots + 1L)))
+  } else {
+    knots <- numeric()
+  }
+
+  # Only construct the data necessary for splines_predict
+  out <- matrix(NA, ncol = degree + length(knots) + intercept, nrow = 1L)
   class(out) <- c("bs", "basis", "matrix")
-  attr(out, "knots") <- attr(bs_obj, "knots")[]
-  attr(out, "degree") <- attr(bs_obj, "degree")
-  attr(out, "Boundary.knots") <- attr(bs_obj, "Boundary.knots")
-  attr(out, "intercept") <- attr(bs_obj, "intercept")
+  attr(out, "knots") <- knots
+  attr(out, "Boundary.knots") <- boundary
+  attr(out, "intercept") <- intercept
+  attr(out, "degree") <- degree
   out
 }
 
+bs_predict <- function(object, x) {
+  xu <- unique(x)
+  ru <- predict(object, xu)
+  res <- ru[match(x, xu), ]
+  copy_attrs <- c("class", "degree", "knots", "Boundary.knots", "intercept")
+  attributes(res)[copy_attrs] <- attributes(ru)[copy_attrs]
+  res
+}
+
 #' @export
 prep.step_bs <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
-  check_type(training[, col_names])
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("double", "integer", "datetime"))
 
   opt <- x$options
   opt$df <- x$deg_free
   opt$degree <- x$degree
-  obj <- lapply(training[, col_names], bs_wrapper, opt)
-  for (i in seq(along.with = col_names))
+  obj <- lapply(training[, col_names], bs_statistics, opt)
+
+  for (i in seq(along.with = col_names)) {
     attr(obj[[i]], "var") <- col_names[i]
+  }
+
   step_bs_new(
     terms = x$terms,
     role = x$role,
@@ -138,6 +158,8 @@ prep.step_bs <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_bs <- function(object, new_data, ...) {
+  check_new_data(names(object$objects), object, new_data)
+
   ## pre-allocate a matrix for the basis functions.
   new_cols <- vapply(object$objects, ncol, c(int = 1L))
   bs_values <-
@@ -148,7 +170,7 @@ bake.step_bs <- function(object, new_data, ...) {
     cols <- (strt):(strt + new_cols[i] - 1)
     orig_var <- attr(object$objects[[i]], "var")
     bs_values[, cols] <-
-      predict(object$objects[[i]], getElement(new_data, i))
+      bs_predict(object$objects[[i]], getElement(new_data, i))
     new_names <-
       paste(orig_var, "bs", names0(new_cols[i], ""), sep = "_")
     colnames(bs_values)[cols] <- new_names
@@ -156,36 +178,32 @@ bake.step_bs <- function(object, new_data, ...) {
     new_data[, orig_var] <- NULL
   }
   new_data <- bind_cols(new_data, as_tibble(bs_values))
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 
 print.step_bs <-
   function(x, width = max(20, options()$width - 28), ...) {
-    cat("B-Splines on ")
-    printer(names(x$objects), x$terms, x$trained, width = width)
+    title <- "B-splines on "
+    print_step(names(x$objects), x$terms, x$trained, title, width)
     invisible(x)
   }
 
-#' @rdname step_bs
-#' @param x A `step_bs` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_bs <- function(x, ...) {
   if (is_trained(x)) {
-    cols <- tibble(terms = names(x$objects))
+    res <- tibble(terms = names(x$objects))
   } else {
     cols <- sel2char(x$terms)
+    res <- tibble(terms = cols)
   }
-  res <- expand.grid(terms = cols, stringsAsFactors = FALSE)
   res$id <- x$id
   as_tibble(res)
 }
 
 # ------------------------------------------------------------------------------
 
-#' @rdname tunable.step
 #' @export
 tunable.step_bs <- function(x, ...) {
   tibble::tibble(
