@@ -1,44 +1,30 @@
-#' Dummy Variables Creation
+#' Create traditional dummy variables
 #'
-#' `step_dummy` creates a *specification* of a recipe
+#' `step_dummy()` creates a *specification* of a recipe
 #'  step that will convert nominal data (e.g. character or factors)
 #'  into one or more numeric binary model terms for the levels of
 #'  the original data.
 #'
+#' @inheritParams step_pca
 #' @inheritParams step_center
-#' @inherit step_center return
-#' @param ... One or more selector functions to choose which
-#'  _factor_ variables will be used to create the dummy variables. See
-#'  [selections()] for more details. The selected
-#'  variables must be factors. For the `tidy` method, these are
-#'  not currently used.
-#' @param role For model terms created by this step, what analysis
-#'  role should they be assigned?. By default, the function assumes
-#'  that the binary dummy variable columns created by the original
-#'  variables will be used as predictors in a model.
+#' @param ... One or more selector functions to choose variables
+#'  for this step. See [selections()] for more details. The selected
+#'  variables _must_ be factors.
 #' @param one_hot A logical. For C levels, should C dummy variables be created
 #' rather than C-1?
-#' @param preserve A single logical; should the selected column(s) be retained
-#'  (in addition to the new dummy variables).
+#' @param preserve Use `keep_original_cols` to specify whether the selected
+#'  column(s) should be retained (in addition to the new dummy variables).
 #' @param naming A function that defines the naming convention for
 #'  new dummy columns. See Details below.
 #' @param levels A list that contains the information needed to
 #'  create dummy variables for each variable contained in
 #'  `terms`. This is `NULL` until the step is trained by
-#'  [prep.recipe()].
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` (the
-#'  selectors or original variables selected) and `columns` (the
-#'  list of corresponding binary columns).
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept dummy_variables
-#' @concept model_specification
-#' @concept dummy_variables
-#' @concept variable_encodings
+#'  [prep()].
+#' @template step-return
+#' @family dummy variable and encoding steps
+#' @seealso [dummy_names()]
 #' @export
-#' @details `step_dummy` will create a set of binary dummy
+#' @details `step_dummy()` will create a set of binary dummy
 #'  variables from a factor variable. For example, if an unordered
 #'  factor column in the data set has levels of "red", "green",
 #'  "blue", the dummy variable bake will create two additional
@@ -50,92 +36,110 @@
 #'  cell) will correspond to the first level of the unordered
 #'  factor being converted.
 #'
-#' The function allows for non-standard naming of the resulting
-#'  variables. For an unordered factor named `x`, with levels `"a"`
-#'  and `"b"`, the default naming convention would be to create a
-#'  new variable called `x_b`. Note that if the factor levels are
-#'  not valid variable names (e.g. "some text with spaces"), it will
-#'  be changed by [base::make.names()] to be valid (see the example
-#'  below). The naming format can be changed using the `naming`
-#'  argument and the function [dummy_names()] is the default. This
-#'  function will also change the names of ordinal dummy variables.
-#'  Instead of values such as "`.L`", "`.Q`", or "`^4`", ordinal
-#'  dummy variables are given simple integer suffixes such as
-#'  "`_1`", "`_2`", etc.
+#' @template dummy-naming
+#'
+#' @details
 #'
 #' To change the type of contrast being used, change the global
 #' contrast option via `options`.
 #'
 #' When the factor being converted has a missing value, all of the
-#'  corresponding dummy variables are also missing.
+#'  corresponding dummy variables are also missing. See [step_unknown()] for
+#'  a solution.
 #'
 #' When data to be processed contains novel levels (i.e., not
 #' contained in the training set), a missing value is assigned to
 #' the results. See [step_other()] for an alternative.
 #'
 #' If no columns are selected (perhaps due to an earlier `step_zv()`),
-#'  `bake()` will return the data as-is (e.g. with no dummy variables).
+#'  [bake()] will return the data as-is (e.g. with no dummy variables).
+#'
+#' Note that, by default, the new dummy variable column names obey the naming
+#' rules for columns. If there are levels such as "0", [dummy_names()] will put
+#' a leading "X" in front of the level (since it uses [make.names()]). This can
+#' be changed by passing in a different function to the `naming` argument for
+#' this step.
+#'
+#' Also, there are a number of contrast methods that return fractional values.
+#' The columns returned by this step are doubles (not integers).
 #'
 #' The [package vignette for dummy variables](https://recipes.tidymodels.org/articles/Dummies.html)
 #' and interactions has more information.
 #'
-#' @seealso [step_factor2string()], [step_string2factor()],
-#'  [dummy_names()], [step_regex()], [step_count()],
-#'  [step_ordinalscore()], [step_unorder()], [step_other()]
-#'  [step_novel()]
-#' @examples
-#' library(modeldata)
-#' data(okc)
-#' okc <- okc[complete.cases(okc),]
+#'  # Tidying
 #'
-#' rec <- recipe(~ diet + age + height, data = okc)
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#'  `terms` (the selectors or original variables selected) and `columns`
+#'  (the list of corresponding binary columns) is returned.
 #'
-#' dummies <- rec %>% step_dummy(diet)
-#' dummies <- prep(dummies, training = okc)
+#' @template case-weights-not-supported
 #'
-#' dummy_data <- bake(dummies, new_data = okc)
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(Sacramento, package = "modeldata")
 #'
-#' unique(okc$diet)
-#' grep("^diet", names(dummy_data), value = TRUE)
+#' # Original data: city has 37 levels
+#' length(unique(Sacramento$city))
+#' unique(Sacramento$city) %>% sort()
 #'
-#' # Obtain the full set of dummy variables using `one_hot` option
-#' rec %>%
-#'   step_dummy(diet, one_hot = TRUE) %>%
-#'   prep(training = okc) %>%
-#'   bake(new_data = NULL, starts_with("diet")) %>%
-#'   names() %>%
-#'   length()
+#' rec <- recipe(~ city + sqft + price, data = Sacramento)
 #'
-#' length(unique(okc$diet))
+#' # Default dummy coding: 36 dummy variables
+#' dummies <- rec %>%
+#'   step_dummy(city) %>%
+#'   prep(training = Sacramento)
 #'
-#' # Without one_hot
-#' length(grep("^diet", names(dummy_data), value = TRUE))
+#' dummy_data <- bake(dummies, new_data = NULL)
+#'
+#' dummy_data %>%
+#'   select(starts_with("city")) %>%
+#'   names() # level "anything" is the reference level
+#'
+#' # Obtain the full set of 37 dummy variables using `one_hot` option
+#' dummies_one_hot <- rec %>%
+#'   step_dummy(city, one_hot = TRUE) %>%
+#'   prep(training = Sacramento)
+#'
+#' dummy_data_one_hot <- bake(dummies_one_hot, new_data = NULL)
+#'
+#' dummy_data_one_hot %>%
+#'   select(starts_with("city")) %>%
+#'   names() # no reference level
 #'
 #'
 #' tidy(dummies, number = 1)
-
-
+#' tidy(dummies_one_hot, number = 1)
 step_dummy <-
   function(recipe,
            ...,
            role = "predictor",
            trained = FALSE,
            one_hot = FALSE,
-           preserve = FALSE,
+           preserve = deprecated(),
            naming = dummy_names,
            levels = NULL,
+           keep_original_cols = FALSE,
            skip = FALSE,
            id = rand_id("dummy")) {
+    if (lifecycle::is_present(preserve)) {
+      lifecycle::deprecate_stop(
+        "0.1.16",
+        "step_dummy(preserve = )",
+        "step_dummy(keep_original_cols = )"
+      )
+      keep_original_cols <- preserve
+    }
+
     add_step(
       recipe,
       step_dummy_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         one_hot = one_hot,
-        preserve = preserve,
+        preserve = keep_original_cols,
         naming = naming,
         levels = levels,
+        keep_original_cols = keep_original_cols,
         skip = skip,
         id = id
       )
@@ -143,7 +147,8 @@ step_dummy <-
   }
 
 step_dummy_new <-
-  function(terms, role, trained, one_hot, preserve, naming, levels, skip, id) {
+  function(terms, role, trained, one_hot, preserve, naming, levels,
+           keep_original_cols, skip, id) {
     step(
       subclass = "dummy",
       terms = terms,
@@ -153,39 +158,19 @@ step_dummy_new <-
       preserve = preserve,
       naming = naming,
       levels = levels,
+      keep_original_cols = keep_original_cols,
       skip = skip,
       id = id
     )
   }
 
-passover <- function(cmd) {
-  # cat("`step_dummy()` was not able to select any columns. ",
-  #     "No dummy variables will be created.\n")
-} # figure out how to return a warning() without exiting
-
 #' @export
 prep.step_dummy <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("string", "factor", "ordered"))
 
   if (length(col_names) > 0) {
-    fac_check <- vapply(training[, col_names], is.factor, logical(1))
-    if (any(!fac_check))
-      rlang::warn(
-        paste0(
-        "The following variables are not factor vectors and will be ignored: ",
-        paste0("`", names(fac_check)[!fac_check], "`", collapse = ", ")
-        )
-      )
-    col_names <- col_names[fac_check]
-    if (length(col_names) == 0) {
-      rlang::abort(
-        paste0(
-        "The `terms` argument in `step_dummy` did not select ",
-        "any factor columns."
-        )
-      )
-    }
-
+    col_names <- check_factor_vars(training, col_names, "step_dummy")
 
     ## I hate doing this but currently we are going to have
     ## to save the terms object from the original (= training)
@@ -193,15 +178,16 @@ prep.step_dummy <- function(x, training, info = NULL, ...) {
     levels <- vector(mode = "list", length = length(col_names))
     names(levels) <- col_names
     for (i in seq_along(col_names)) {
-      form_chr <- paste0("~", col_names[i])
+      form <- rlang::new_formula(lhs = NULL, rhs = rlang::sym(col_names[i]))
       if (x$one_hot) {
-        form_chr <- paste0(form_chr, "-1")
+        form <- stats::update.formula(form, ~ . -1)
       }
-      form <- as.formula(form_chr)
-      terms <- model.frame(form,
-                           data = training,
-                           xlev = x$levels[[i]],
-                           na.action = na.pass)
+      terms <- model.frame(
+        formula = form,
+        data = training[1, ],
+        xlev = x$levels[[i]],
+        na.action = na.pass
+      )
       levels[[i]] <- attr(terms, "terms")
 
       ## About factor levels here: once dummy variables are made,
@@ -226,26 +212,55 @@ prep.step_dummy <- function(x, training, info = NULL, ...) {
     preserve = x$preserve,
     naming = x$naming,
     levels = levels,
+    keep_original_cols = get_keep_original_cols(x),
     skip = x$skip,
     id = x$id
   )
 }
 
-warn_new_levels <- function(dat, lvl) {
+check_factor_vars <- function(data, col_names, step_name, call = caller_env()) {
+  fac_check <- vapply(data[, col_names], is.factor, logical(1))
+  if (any(!fac_check)) {
+    rlang::warn(
+      paste0(
+        "The following variables are not factor vectors and will be ignored: ",
+        paste0("`", names(fac_check)[!fac_check], "`", collapse = ", ")
+      )
+    )
+  }
+  col_names <- col_names[fac_check]
+  if (length(col_names) == 0) {
+    rlang::abort(
+      paste0(
+        "The `terms` argument in `",
+        step_name,
+        "` did not select ",
+        "any factor columns."
+      ),
+      call = call
+    )
+  }
+  col_names
+}
+
+warn_new_levels <- function(dat, lvl, details = NULL) {
   ind <- which(!(dat %in% lvl))
   if (length(ind) > 0) {
     lvl2 <- unique(dat[ind])
     rlang::warn(
-      paste0("There are new levels in a factor: ",
-            paste0(lvl2, collapse = ", ")
-            )
+      paste0(
+        "There are new levels in a factor: ",
+        paste0(lvl2, collapse = ", "),
+        details
       )
+    )
   }
   invisible(NULL)
 }
 
 #' @export
 bake.step_dummy <- function(object, new_data, ...) {
+  check_new_data(names(object$levels), object, new_data)
 
   # If no terms were selected
   if (length(object$levels) == 0) {
@@ -253,6 +268,7 @@ bake.step_dummy <- function(object, new_data, ...) {
   }
 
   col_names <- names(object$levels)
+  keep_original_cols <- get_keep_original_cols(object)
 
   ## `na.action` cannot be passed to `model.matrix` but we
   ## can change it globally for a bit
@@ -266,13 +282,18 @@ bake.step_dummy <- function(object, new_data, ...) {
     orig_var <- names(object$levels)[i]
     fac_type <- attr(object$levels[[i]], "dataClasses")
 
-    if (!any(names(attributes(object$levels[[i]])) == "values"))
+    if (!any(names(attributes(object$levels[[i]])) == "values")) {
       rlang::abort("Factor level values not recorded")
+    }
 
-    if (length(attr(object$levels[[i]], "values")) == 1)
+    if (length(attr(object$levels[[i]], "values")) == 1) {
       rlang::abort(
-        paste0("Only one factor level in ", orig_var)
+        paste0(
+          "Only one factor level in ", orig_var, ": ",
+          attr(object$levels[[i]], "values")
         )
+      )
+    }
 
     warn_new_levels(
       new_data[[orig_var]],
@@ -281,12 +302,13 @@ bake.step_dummy <- function(object, new_data, ...) {
 
     new_data[, orig_var] <-
       factor(getElement(new_data, orig_var),
-             levels = attr(object$levels[[i]], "values"),
-             ordered = fac_type == "ordered")
+        levels = attr(object$levels[[i]], "values"),
+        ordered = fac_type == "ordered"
+      )
 
     indicators <-
       model.frame(
-        as.formula(paste0("~", orig_var)),
+        rlang::new_formula(lhs = NULL, rhs = rlang::sym(orig_var)),
         data = new_data[, orig_var],
         xlev = attr(object$levels[[i]], "values"),
         na.action = na.pass
@@ -307,53 +329,39 @@ bake.step_dummy <- function(object, new_data, ...) {
     }
 
     ## use backticks for nonstandard factor levels here
-    used_lvl <- gsub(paste0("^", col_names[i]), "", colnames(indicators))
+    used_lvl <- gsub(paste0("^\\`?", col_names[i], "\\`?"), "", colnames(indicators))
     colnames(indicators) <- object$naming(col_names[i], used_lvl, fac_type == "ordered")
     new_data <- bind_cols(new_data, as_tibble(indicators))
-    if (!object$preserve) {
+    if (any(!object$preserve, !keep_original_cols)) {
       new_data[, col_names[i]] <- NULL
     }
   }
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 print.step_dummy <-
   function(x, width = max(20, options()$width - 20), ...) {
-    if (x$trained) {
-      if (length(x$levels) > 0) {
-        cat("Dummy variables from ")
-        cat(format_ch_vec(names(x$levels), width = width))
-      } else {
-        cat("Dummy variables were *not* created since no columns were selected.")
-      }
-    } else {
-      cat("Dummy variables from ", sep = "")
-      cat(format_selectors(x$terms, width = width))
-    }
-    if (x$trained)
-      cat(" [trained]\n")
-    else
-      cat("\n")
+    title <- "Dummy variables from "
+    print_step(names(x$levels), x$terms, x$trained, title, width)
     invisible(x)
   }
 
 
-get_dummy_columns <- function(x) {
-  tibble(columns = attr(x, "values"))
+get_dummy_columns <- function(x, one_hot) {
+  x <- attr(x, "values")
+  if (!one_hot) x <- x[-1]
+  tibble(columns = x)
 }
 
 
-#' @rdname step_dummy
-#' @param x A `step_dummy` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_dummy <- function(x, ...) {
   if (is_trained(x)) {
     if (length(x$levels) > 0) {
-      res <- purrr::map_dfr(x$levels, get_dummy_columns, .id = "terms")
+      res <- purrr::map_dfr(x$levels, get_dummy_columns, x$one_hot, .id = "terms")
     } else {
-      res <- tibble(terms = rlang::na_chr, columns = rlang::na_chr)
+      res <- tibble(terms = character(), columns = character())
     }
   } else {
     res <- tibble(terms = sel2char(x$terms), columns = rlang::na_chr)
