@@ -24,6 +24,6 @@ format_selectors <- function(x, width = options()$width - 9) {
     expr_deparse(quo_get_expr(x))
   })
 
-  x_items <- unlist(x_items)
+  x_items <- vctrs::list_unchop(x_items, ptype = character())
   format_ch_vec(x_items, width = width, sep = ", ")
 }
