@@ -17,23 +17,26 @@
 #' formula(iris_rec)
 #' @export
 formula.recipe <- function(x, ...) {
-  if (!fully_trained(x))
+  if (!fully_trained(x)) {
     rlang::abort(
       paste0(
-      "The recipe must be prepped before the ",
-      "formula can be computed."
+        "The recipe must be prepped before the ",
+        "formula can be computed."
       )
     )
+  }
 
   x <- summary(x)
   x_vars <- x$variable[x$role == "predictor"]
   x_vars <- x_vars[!is.na(x_vars)]
-  if (length(x_vars) == 0)
+  if (length(x_vars) == 0) {
     x_vars <- 1
+  }
   y_vars <- x$variable[x$role == "outcome"]
   y_vars <- y_vars[!is.na(y_vars)]
-  if (length(y_vars) == 0)
+  if (length(y_vars) == 0) {
     y_vars <- ""
+  }
   x_vars <- paste0(x_vars, collapse = "+")
   y_vars <- paste0(y_vars, collapse = "+")
 
