@@ -11,7 +11,9 @@
 #'  **Note** that the arguments `x` and `features` should not be passed here
 #'  (or at all).
 #' @param res An S4 [kernlab::kpca()] object is stored here once this
-#'  preprocessing step has be trained by [`prep()`][prep.recipe()].
+#'  preprocessing step has be trained by [prep()].
+#' @param columns A character string of variable names that will
+#'  be populated elsewhere.
 #' @template step-return
 #' @family multivariate transformation steps
 #' @export
@@ -26,57 +28,62 @@
 #'
 #' @template kpca-info
 #'
-#' @examples
-#' library(modeldata)
-#' data(biomass)
+#' @template case-weights-not-supported
 #'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#' @examplesIf rlang::is_installed(c("modeldata", "ggplot2", "kernlab"))
+#' library(ggplot2)
+#' data(biomass, package = "modeldata")
 #'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-#'               data = biomass_tr)
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
+#'
+#' rec <- recipe(
+#'   HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
+#'   data = biomass_tr
+#' )
 #'
 #' kpca_trans <- rec %>%
 #'   step_YeoJohnson(all_numeric_predictors()) %>%
 #'   step_normalize(all_numeric_predictors()) %>%
 #'   step_kpca(all_numeric_predictors())
 #'
-#' if (require(dimRed) & require(kernlab)) {
-#'   kpca_estimates <- prep(kpca_trans, training = biomass_tr)
+#' kpca_estimates <- prep(kpca_trans, training = biomass_tr)
 #'
-#'   kpca_te <- bake(kpca_estimates, biomass_te)
+#' kpca_te <- bake(kpca_estimates, biomass_te)
 #'
-#'   rng <- extendrange(c(kpca_te$kPC1, kpca_te$kPC2))
-#'   plot(kpca_te$kPC1, kpca_te$kPC2,
-#'        xlim = rng, ylim = rng)
+#' ggplot(kpca_te, aes(x = kPC1, y = kPC2)) +
+#'   geom_point() +
+#'   coord_equal()
 #'
-#'   tidy(kpca_trans, number = 3)
-#'   tidy(kpca_estimates, number = 3)
-#' }
+#' tidy(kpca_trans, number = 3)
+#' tidy(kpca_estimates, number = 3)
 step_kpca <-
   function(recipe,
            ...,
            role = "predictor",
            trained = FALSE,
-           num_comp  = 5,
+           num_comp = 5,
            res = NULL,
-           options = list(kernel = "rbfdot",
-                          kpar = list(sigma = 0.2)),
+           columns = NULL,
+           options = list(
+             kernel = "rbfdot",
+             kpar = list(sigma = 0.2)
+           ),
            prefix = "kPC",
            keep_original_cols = FALSE,
            skip = FALSE,
            id = rand_id("kpca")) {
-
     recipes_pkg_check(required_pkgs.step_kpca())
 
     add_step(
       recipe,
       step_kpca_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         num_comp = num_comp,
         res = res,
+        columns = columns,
         options = options,
         prefix = prefix,
         keep_original_cols = keep_original_cols,
@@ -84,10 +91,10 @@ step_kpca <-
         id = id
       )
     )
-}
+  }
 
 step_kpca_new <-
-  function(terms, role, trained, num_comp, res, options, prefix,
+  function(terms, role, trained, num_comp, res, columns, options, prefix,
            keep_original_cols, skip, id) {
     step(
       subclass = "kpca",
@@ -96,6 +103,7 @@ step_kpca_new <-
       trained = trained,
       num_comp = num_comp,
       res = res,
+      columns = columns,
       options = options,
       prefix = prefix,
       keep_original_cols = keep_original_cols,
@@ -107,25 +115,23 @@ step_kpca_new <-
 #' @export
 prep.step_kpca <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
-  check_type(training[, col_names])
+  check_type(training[, col_names], types = c("double", "integer"))
 
-  if (x$num_comp > 0) {
-    kprc <- dimRed::kPCA(stdpars = c(list(ndim = x$num_comp), x$options))
-    kprc <-
-      try(
-        suppressMessages({
-          kprc@fun(
-            dimRed::dimRedData(as.data.frame(training[, col_names, drop = FALSE])),
-            kprc@stdpars
-          )
-        }),
-        silent = TRUE
+  if (x$num_comp > 0 && length(col_names) > 0) {
+    cl <-
+      rlang::call2(
+        "kpca",
+        .ns = "kernlab",
+        x = rlang::expr(as.matrix(training[, col_names])),
+        features = x$num_comp
       )
+    cl <- call_modify(cl, !!!x$options)
+    kprc <- try(rlang::eval_tidy(cl), silent = TRUE)
     if (inherits(kprc, "try-error")) {
       rlang::abort(paste0("`step_kpca` failed with error:\n", as.character(kprc)))
     }
   } else {
-    kprc <- list(x_vars = col_names)
+    kprc <- NULL
   }
 
   step_kpca_new(
@@ -135,6 +141,7 @@ prep.step_kpca <- function(x, training, info = NULL, ...) {
     num_comp = x$num_comp,
     options = x$options,
     res = kprc,
+    columns = col_names,
     prefix = x$prefix,
     keep_original_cols = get_keep_original_cols(x),
     skip = x$skip,
@@ -144,36 +151,35 @@ prep.step_kpca <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_kpca <- function(object, new_data, ...) {
-  if (object$num_comp > 0) {
-    pca_vars <- colnames(environment(object$res@apply)$indata)
-    comps <- object$res@apply(
-      dimRed::dimRedData(as.data.frame(new_data[, pca_vars, drop = FALSE]))
-    )@data
+  uses_dim_red(object)
+
+  if (object$num_comp > 0 && length(object$columns) > 0) {
+    check_new_data(object$columns, object, new_data)
+
+    cl <-
+      rlang::call2(
+        "predict",
+        .ns = "kernlab",
+        object = object$res,
+        rlang::expr(as.matrix(new_data[, object$columns]))
+      )
+    comps <- rlang::eval_tidy(cl)
     comps <- comps[, 1:object$num_comp, drop = FALSE]
+    colnames(comps) <- names0(ncol(comps), object$prefix)
     comps <- check_name(comps, new_data, object)
     new_data <- bind_cols(new_data, as_tibble(comps))
     keep_original_cols <- get_keep_original_cols(object)
 
     if (!keep_original_cols) {
-      new_data <- new_data[, !(colnames(new_data) %in% pca_vars), drop = FALSE]
+      new_data <- new_data[, !(colnames(new_data) %in% object$columns), drop = FALSE]
     }
   }
-  as_tibble(new_data)
+  new_data
 }
 
 print.step_kpca <- function(x, width = max(20, options()$width - 40), ...) {
-  if (x$trained) {
-    if (x$num_comp == 0) {
-      cat("No kPCA components were extracted.\n")
-    } else {
-      cat("Kernel PCA (", x$res@pars$kernel, ") extraction with ", sep = "")
-      cat(format_ch_vec(colnames(x$res@org.data), width = width))
-    }
-  } else {
-    cat("Kernel PCA extraction with ", sep = "")
-    cat(format_selectors(x$terms, width = width))
-  }
-  if (x$trained) cat(" [trained]\n") else cat("\n")
+  title <- "Kernel PCA extraction with "
+  print_step(x$columns, x$terms, x$trained, title, width)
   invisible(x)
 }
 
@@ -181,12 +187,9 @@ print.step_kpca <- function(x, width = max(20, options()$width - 40), ...) {
 #' @rdname tidy.recipe
 #' @export
 tidy.step_kpca <- function(x, ...) {
+  uses_dim_red(x)
   if (is_trained(x)) {
-    if (x$num_comp > 0) {
-      res <- tibble(terms = colnames(x$res@org.data))
-    } else {
-      res <- tibble(terms = unname(x$res$x_vars))
-    }
+    res <- tibble(terms = unname(x$columns))
   } else {
     term_names <- sel2char(x$terms)
     res <- tibble(terms = term_names)
@@ -200,5 +203,5 @@ tidy.step_kpca <- function(x, ...) {
 #' @rdname required_pkgs.recipe
 #' @export
 required_pkgs.step_kpca <- function(x, ...) {
-  c("dimRed", "kernlab")
+  c("kernlab")
 }
