@@ -4,27 +4,18 @@
 #'  that will create new columns that are basis expansions of
 #'  variables using natural splines.
 #'
+#' @inheritParams step_pca
 #' @inheritParams step_center
-#' @param ... One or more selector functions to choose which
-#'  variables are affected by the step. See [selections()]
-#'  for more details. For the `tidy` method, these are not
-#'  currently used.
-#' @param role For model terms created by this step, what analysis
-#'  role should they be assigned?. By default, the function assumes
-#'  that the new columns created from the original variables will be
-#'  used as predictors in a model.
-#' @param deg_free The degrees of freedom.
+#' @param deg_free The degrees of freedom for the natural spline. As the
+#'  degrees of freedom for a natural spline increase, more flexible and
+#'  complex curves can be generated. When a single degree of freedom is used,
+#'  the result is a rescaled version of the original data.
 #' @param objects A list of [splines::ns()] objects
 #'  created once the step has been trained.
 #' @param options A list of options for [splines::ns()]
 #'  which should not include `x` or `df`.
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` which is
-#'  the columns that will be affected and `holiday`.
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept basis_expansion
+#' @template step-return
+#' @family individual transformation steps
 #' @export
 #' @details `step_ns` can create new features from a single variable
 #'  that enable fitting routines to model this variable in a
@@ -33,15 +24,24 @@
 #'  [splines::ns()]. The original variables are removed
 #'  from the data and new columns are added. The naming convention
 #'  for the new variables is `varname_ns_1` and so on.
-#' @examples
-#' library(modeldata)
-#' data(biomass)
 #'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#'  # Tidying
 #'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-#'               data = biomass_tr)
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with column
+#'  `terms` (the columns that will be affected) is returned.
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(biomass, package = "modeldata")
+#'
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
+#'
+#' rec <- recipe(
+#'   HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
+#'   data = biomass_tr
+#' )
 #'
 #' with_splines <- rec %>%
 #'   step_ns(carbon, hydrogen)
@@ -49,9 +49,6 @@
 #'
 #' expanded <- bake(with_splines, biomass_te)
 #' expanded
-#' @seealso [step_poly()] [recipe()]
-#'   [prep.recipe()] [bake.recipe()]
-
 step_ns <-
   function(recipe,
            ...,
@@ -65,7 +62,7 @@ step_ns <-
     add_step(
       recipe,
       step_ns_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         trained = trained,
         deg_free = deg_free,
         role = role,
@@ -92,32 +89,52 @@ step_ns_new <-
     )
   }
 
+ns_statistics <- function(x, args) {
+  # Only do the parameter computations from splines::bs() / splines::ns(), don't evaluate at x.
+  degree <- 1L
+  intercept <- as.logical(args$intercept %||% FALSE)
+  # This behaves differently from splines::ns() if length(x) is 1
+  boundary <- sort(args$Boundary.knots) %||% range(x)
 
-ns_wrapper <- function(x, args) {
-  if (!("Boundary.knots" %in% names(args)))
-    args$Boundary.knots <- range(x)
-  args$x <- x
-  ns_obj <- do.call("ns", args)
-  ## don't need to save the original data so keep 1 row
-  out <- matrix(NA, ncol = ncol(ns_obj), nrow = 1)
+  # This behaves differently from splines::bs() and splines::ns() if num_knots < 0L
+  # the original implementations issue a warning.
+  if (!is.null(args$df) && is.null(args$knots) && args$df - degree - intercept >= 1L) {
+    num_knots <- args$df - degree - intercept
+    ok <- !is.na(x) & x >= boundary[1L] & x <= boundary[2L]
+    knots <- unname(quantile(x[ok], seq_len(num_knots) / (num_knots + 1L)))
+  } else {
+    knots <- numeric()
+  }
+
+  # Only construct the data necessary for splines_predict
+  out <- matrix(NA, ncol = degree + length(knots) + intercept, nrow = 1L)
   class(out) <- c("ns", "basis", "matrix")
-  attr(out, "knots") <- attr(ns_obj, "knots")[]
-  attr(out, "Boundary.knots") <- attr(ns_obj, "Boundary.knots")
-  attr(out, "intercept") <- attr(ns_obj, "intercept")
+  attr(out, "knots") <- knots
+  attr(out, "Boundary.knots") <- boundary
+  attr(out, "intercept") <- intercept
   out
 }
 
+ns_predict <- function(object, x) {
+  xu <- unique(x)
+  ru <- predict(object, xu)
+  res <- ru[match(x, xu), ]
+  copy_attrs <- c("class", "knots", "Boundary.knots", "intercept")
+  attributes(res)[copy_attrs] <- attributes(ru)[copy_attrs]
+  res
+}
+
 #' @export
 prep.step_ns <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
-
-  check_type(training[, col_names])
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("double", "integer", "datetime"))
 
   opt <- x$options
   opt$df <- x$deg_free
-  obj <- lapply(training[, col_names], ns_wrapper, opt)
-  for (i in seq(along.with = col_names))
+  obj <- lapply(training[, col_names], ns_statistics, opt)
+  for (i in seq(along.with = col_names)) {
     attr(obj[[i]], "var") <- col_names[i]
+  }
   step_ns_new(
     terms = x$terms,
     role = x$role,
@@ -132,6 +149,7 @@ prep.step_ns <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_ns <- function(object, new_data, ...) {
+  check_new_data(names(object$objects), object, new_data)
   ## pre-allocate a matrix for the basis functions.
   new_cols <- vapply(object$objects, ncol, c(int = 1L))
   ns_values <-
@@ -142,7 +160,7 @@ bake.step_ns <- function(object, new_data, ...) {
     cols <- (strt):(strt + new_cols[i] - 1)
     orig_var <- attr(object$objects[[i]], "var")
     ns_values[, cols] <-
-      predict(object$objects[[i]], getElement(new_data, i))
+      ns_predict(object$objects[[i]], getElement(new_data, i))
     new_names <-
       paste(orig_var, "ns", names0(new_cols[i], ""), sep = "_")
     colnames(ns_values)[cols] <- new_names
@@ -150,36 +168,28 @@ bake.step_ns <- function(object, new_data, ...) {
     new_data[, orig_var] <- NULL
   }
   new_data <- bind_cols(new_data, as_tibble(ns_values))
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 
 print.step_ns <-
   function(x, width = max(20, options()$width - 28), ...) {
-    cat("Natural Splines on ")
-    printer(names(x$objects), x$terms, x$trained, width = width)
+    title <- "Natural splines on "
+    print_step(names(x$objects), x$terms, x$trained, title, width)
     invisible(x)
   }
 
-#' @rdname step_ns
-#' @param x A `step_ns` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_ns <- function(x, ...) {
   if (is_trained(x)) {
-    cols <- tibble(terms = names(x$objects))
+    terms <- names(x$objects)
   } else {
-    cols <- sel2char(x$terms)
+    terms <- sel2char(x$terms)
   }
-  res <- expand.grid(terms = cols, stringsAsFactors = FALSE)
-  res$id <- x$id
-  as_tibble(res)
+  tibble(terms = terms, id = x$id)
 }
 
-
-
-#' @rdname tunable.step
 #' @export
 tunable.step_ns <- function(x, ...) {
   tibble::tibble(
