@@ -11,8 +11,9 @@
 #'  below).
 #' @param removals A character string that contains the names of
 #'  columns that should be removed. These values are not determined
-#'  until [prep.recipe()] is called.
+#'  until [prep()] is called.
 #' @template step-return
+#' @template filter-steps
 #' @family variable filter steps
 #' @export
 #'
@@ -40,21 +41,25 @@
 #' In the above example, the frequency ratio is 999 and the unique
 #'  value percent is 0.2%.
 #'
-#' When you [`tidy()`] this step, a tibble with column `terms` (the columns
-#'  that will be removed) is returned.
+#' # Tidying
 #'
-#' @examples
-#' library(modeldata)
-#' data(biomass)
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with column
+#' `terms` (the columns that will be removed) is returned.
+#'
+#' @template case-weights-unsupervised
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(biomass, package = "modeldata")
 #'
 #' biomass$sparse <- c(1, rep(0, nrow(biomass) - 1))
 #'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
 #'
 #' rec <- recipe(HHV ~ carbon + hydrogen + oxygen +
-#'                     nitrogen + sulfur + sparse,
-#'               data = biomass_tr)
+#'   nitrogen + sulfur + sparse,
+#' data = biomass_tr
+#' )
 #'
 #' nzv_filter <- rec %>%
 #'   step_nzv(all_predictors())
@@ -77,7 +82,6 @@ step_nzv <-
            removals = NULL,
            skip = FALSE,
            id = rand_id("nzv")) {
-
     exp_list <- list(freq_cut = 95 / 5, unique_cut = 10)
     if (!isTRUE(all.equal(exp_list, options))) {
       freq_cut <- options$freq_cut
@@ -92,7 +96,7 @@ step_nzv <-
     add_step(
       recipe,
       step_nzv_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         freq_cut = freq_cut,
@@ -100,13 +104,15 @@ step_nzv <-
         options = options,
         removals = removals,
         skip = skip,
-        id = id
+        id = id,
+        case_weights = NULL
       )
     )
   }
 
 step_nzv_new <-
-  function(terms, role, trained, freq_cut, unique_cut, options, removals, skip, id) {
+  function(terms, role, trained, freq_cut, unique_cut, options,
+           removals, skip, id, case_weights) {
     step(
       subclass = "nzv",
       terms = terms,
@@ -117,7 +123,8 @@ step_nzv_new <-
       options = options,
       removals = removals,
       skip = skip,
-      id = id
+      id = id,
+      case_weights = case_weights
     )
   }
 
@@ -125,8 +132,15 @@ step_nzv_new <-
 prep.step_nzv <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
 
+  wts <- get_case_weights(info, training)
+  were_weights_used <- are_weights_used(wts, unsupervised = TRUE)
+  if (isFALSE(were_weights_used)) {
+    wts <- NULL
+  }
+
   filter <- nzv(
     x = training[, col_names],
+    wts = wts,
     freq_cut = x$freq_cut,
     unique_cut = x$unique_cut
   )
@@ -140,44 +154,41 @@ prep.step_nzv <- function(x, training, info = NULL, ...) {
     options = x$options,
     removals = filter,
     skip = x$skip,
-    id = x$id
+    id = x$id,
+    case_weights = were_weights_used
   )
 }
 
 #' @export
 bake.step_nzv <- function(object, new_data, ...) {
-  if (length(object$removals) > 0)
+  if (length(object$removals) > 0) {
     new_data <- new_data[, !(colnames(new_data) %in% object$removals)]
-  as_tibble(new_data)
+  }
+  new_data
 }
 
 print.step_nzv <-
   function(x, width = max(20, options()$width - 38), ...) {
     if (x$trained) {
-      if (length(x$removals) > 0) {
-        cat("Sparse, unbalanced variable filter removed ")
-        cat(format_ch_vec(x$removals, width = width))
-      } else
-        cat("Sparse, unbalanced variable filter removed no terms")
+      title <- "Sparse, unbalanced variable filter removed "
     } else {
-      cat("Sparse, unbalanced variable filter on ", sep = "")
-      cat(format_selectors(x$terms, width = width))
+      title <- "Sparse, unbalanced variable filter on "
     }
-    if (x$trained)
-      cat(" [trained]\n")
-    else
-      cat("\n")
+    print_step(x$removals, x$terms, x$trained, title, width,
+               case_weights = x$case_weights)
     invisible(x)
   }
 
 nzv <- function(x,
+                wts,
                 freq_cut = 95 / 5,
                 unique_cut = 10) {
-  if (is.null(dim(x)))
+  if (is.null(dim(x))) {
     x <- matrix(x, ncol = 1)
+  }
 
   fr_foo <- function(data) {
-    t <- table(data[!is.na(data)])
+    t <- weighted_table(data[!is.na(data)], wts = wts)
     if (length(t) <= 1) {
       return(0)
     }
@@ -187,18 +198,20 @@ nzv <- function(x,
   }
 
   freq_ratio <- vapply(x, fr_foo, c(ratio = 0))
-  uni_foo <- function(data)
+  uni_foo <- function(data) {
     length(unique(data[!is.na(data)]))
+  }
   lunique <- vapply(x, uni_foo, c(num = 0))
   pct_unique <- 100 * lunique / vapply(x, length, c(num = 0))
 
-  zero_func <- function(data)
+  zero_func <- function(data) {
     all(is.na(data))
+  }
   zero_var <- (lunique == 1) | vapply(x, zero_func, c(zv = TRUE))
 
   out <-
-    which( (freq_ratio > freq_cut &
-             pct_unique <= unique_cut) | zero_var)
+    which((freq_ratio > freq_cut &
+      pct_unique <= unique_cut) | zero_var)
   names(out) <- NULL
   colnames(x)[out]
 }
@@ -207,8 +220,6 @@ nzv <- function(x,
 #' @export
 tidy.step_nzv <- tidy_filter
 
-
-#' @rdname tunable.recipe
 #' @export
 tunable.step_nzv <- function(x, ...) {
   tibble::tibble(
