@@ -14,7 +14,7 @@
 #' @param other A single character value for the "other" category.
 #' @param objects A list of objects that contain the information
 #'  to pool infrequent levels that is determined by
-#'  [prep.recipe()].
+#'  [prep()].
 #' @template step-return
 #' @family dummy variable and encoding steps
 #' @seealso [dummy_names()]
@@ -26,7 +26,7 @@
 #'
 #' If no pooling is done the data are unmodified (although character data may
 #'   be changed to factors based on the value of `strings_as_factors` in
-#'   [prep.recipe()]). Otherwise, a factor is always returned with
+#'   [prep()]). Otherwise, a factor is always returned with
 #'   different factor levels.
 #'
 #' If `threshold` is less than the largest category proportion, all levels
@@ -42,49 +42,51 @@
 #' When data to be processed contains novel levels (i.e., not
 #' contained in the training set), the other category is assigned.
 #'
-#' When you [`tidy()`] this step, a tibble with columns `terms` (the
-#'  columns that will be affected) and `retained` (the factor
-#'  levels that were not pulled into "other") is returned.
+#' # Tidying
 #'
-#' @examples
-#' library(modeldata)
-#' data(okc)
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#' `terms` (the columns that will be affected) and `retained` (the factor
+#' levels that were not pulled into "other") is returned.
+#'
+#' @template case-weights-unsupervised
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(Sacramento, package = "modeldata")
 #'
 #' set.seed(19)
-#' in_train <- sample(1:nrow(okc), size = 30000)
+#' in_train <- sample(1:nrow(Sacramento), size = 800)
 #'
-#' okc_tr <- okc[ in_train,]
-#' okc_te <- okc[-in_train,]
+#' sacr_tr <- Sacramento[in_train, ]
+#' sacr_te <- Sacramento[-in_train, ]
 #'
-#' rec <- recipe(~ diet + location, data = okc_tr)
+#' rec <- recipe(~ city + zip, data = sacr_tr)
 #'
 #'
 #' rec <- rec %>%
-#'   step_other(diet, location, threshold = .1, other = "other values")
-#' rec <- prep(rec, training = okc_tr)
+#'   step_other(city, zip, threshold = .1, other = "other values")
+#' rec <- prep(rec, training = sacr_tr)
 #'
-#' collapsed <- bake(rec, okc_te)
-#' table(okc_te$diet, collapsed$diet, useNA = "always")
+#' collapsed <- bake(rec, sacr_te)
+#' table(sacr_te$city, collapsed$city, useNA = "always")
 #'
 #' tidy(rec, number = 1)
 #'
 #' # novel levels are also "othered"
-#' tahiti <- okc[1,]
-#' tahiti$location <- "a magical place"
+#' tahiti <- Sacramento[1, ]
+#' tahiti$zip <- "a magical place"
 #' bake(rec, tahiti)
 #'
 #' # threshold as a frequency
-#' rec <- recipe(~ diet + location, data = okc_tr)
+#' rec <- recipe(~ city + zip, data = sacr_tr)
 #'
 #' rec <- rec %>%
-#'   step_other(diet, location, threshold = 2000, other = "other values")
-#' rec <- prep(rec, training = okc_tr)
+#'   step_other(city, zip, threshold = 2000, other = "other values")
+#' rec <- prep(rec, training = sacr_tr)
 #'
 #' tidy(rec, number = 1)
 #' # compare it to
-#' # okc_tr %>% count(diet, sort = TRUE) %>% top_n(4)
-#' # okc_tr %>% count(location, sort = TRUE) %>% top_n(3)
-
+#' # sacr_tr %>% count(city, sort = TRUE) %>% top_n(4)
+#' # sacr_tr %>% count(zip, sort = TRUE) %>% top_n(3)
 step_other <-
   function(recipe,
            ...,
@@ -96,8 +98,8 @@ step_other <-
            skip = FALSE,
            id = rand_id("other")) {
     if (!is_tune(threshold) & !is_varying(threshold)) {
-      if (threshold <= 0) {
-        rlang::abort("`threshold` should be greater than zero")
+      if (threshold < 0) {
+        rlang::abort("`threshold` should be non-negative.")
       }
       if (threshold >= 1 && !is_integerish(threshold)) {
         rlang::abort("If `threshold` is greater than one it should be an integer.")
@@ -106,20 +108,22 @@ step_other <-
     add_step(
       recipe,
       step_other_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         threshold = threshold,
         other = other,
         objects = objects,
         skip = skip,
-        id = id
+        id = id,
+        case_weights = NULL
       )
     )
   }
 
 step_other_new <-
-  function(terms, role, trained, threshold, other, objects, skip, id) {
+  function(terms, role, trained, threshold, other, objects, skip, id,
+           case_weights) {
     step(
       subclass = "other",
       terms = terms,
@@ -129,23 +133,28 @@ step_other_new <-
       other = other,
       objects = objects,
       skip = skip,
-      id = id
+      id = id,
+      case_weights = case_weights
     )
   }
 
 #' @export
 prep.step_other <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("string", "factor", "ordered"))
 
-  if (length(col_names) > 0) {
-    objects <- lapply(training[, col_names],
-                      keep_levels,
-                      threshold = x$threshold,
-                      other = x$other)
-  } else {
-    objects <- NULL
+  wts <- get_case_weights(info, training)
+  were_weights_used <- are_weights_used(wts, unsupervised = TRUE)
+  if (isFALSE(were_weights_used)) {
+    wts <- NULL
   }
 
+  objects <- lapply(training[, col_names],
+                    keep_levels,
+                    threshold = x$threshold,
+                    other = x$other,
+                    wts = wts)
+
   step_other_new(
     terms = x$terms,
     role = x$role,
@@ -154,93 +163,102 @@ prep.step_other <- function(x, training, info = NULL, ...) {
     other = x$other,
     objects = objects,
     skip = x$skip,
-    id = x$id
+    id = x$id,
+    case_weights = were_weights_used
   )
 }
 
 #' @export
 bake.step_other <- function(object, new_data, ...) {
-  if (!is.null(object$objects)) {
-    for (i in names(object$objects)) {
-      if (object$objects[[i]]$collapse) {
-        tmp <- if (!is.character(new_data[, i]))
-          as.character(getElement(new_data, i))
-        else
-          getElement(new_data, i)
-
-        tmp <- ifelse(
-          !(tmp %in% object$objects[[i]]$keep) & !is.na(tmp),
-          object$objects[[i]]$other,
-          tmp
-        )
-
-        # assign other factor levels other here too.
-        tmp <- factor(tmp,
-                      levels = c(object$objects[[i]]$keep,
-                                 object$objects[[i]]$other))
-
-        new_data[, i] <- tmp
+  check_new_data(names(object$objects), object, new_data)
+  for (i in names(object$objects)) {
+    if (object$objects[[i]]$collapse) {
+      tmp <- if (!is.character(new_data[, i])) {
+        as.character(getElement(new_data, i))
+      } else {
+        getElement(new_data, i)
       }
+
+      tmp <- ifelse(
+        !(tmp %in% object$objects[[i]]$keep) & !is.na(tmp),
+        object$objects[[i]]$other,
+        tmp
+      )
+
+      # assign other factor levels other here too.
+      tmp <- factor(tmp,
+        levels = c(
+          object$objects[[i]]$keep,
+          object$objects[[i]]$other
+        )
+      )
+
+      new_data[, i] <- tmp
     }
   }
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 print.step_other <-
   function(x, width = max(20, options()$width - 30), ...) {
-
+    title <- "Collapsing factor levels for "
     if (x$trained) {
-      collapsed <- map_lgl(x$objects, ~ .x$collapse)
-      collapsed <- names(collapsed)[collapsed]
-      if (length(collapsed) > 0) {
-        cat("Collapsing factor levels for ", sep = "")
-        printer(collapsed, x$terms, x$trained, width = width)
-      } else {
-        cat("No factor levels were collapsed\n")
-      }
+      columns <- map_lgl(x$objects, ~ .x$collapse)
+      columns <- names(columns)[columns]
     } else {
-      cat("Collapsing factor levels for ", sep = "")
-      printer(names(x$objects), x$terms, x$trained, width = width)
+      columns <- names(x$objects)
     }
+    print_step(columns, x$terms, x$trained, title, width,
+               case_weights = x$case_weights)
     invisible(x)
   }
 
-keep_levels <- function(x, threshold = .1, other = "other") {
-  if (!is.factor(x))
+keep_levels <- function(x, threshold = .1, other = "other", wts = NULL,
+                        call = caller_env(2)) {
+  if (!is.factor(x)) {
     x <- factor(x)
+  }
 
-  xtab <- sort(table(x, useNA = "no"), decreasing = TRUE)
+  xtab <- sort(weighted_table(x, wts = wts), decreasing = TRUE)
 
   if (threshold < 1) {
-    xtab <- xtab / sum(!is.na(x))
+    if (is.null(wts)) {
+        xtab <- xtab / sum(!is.na(x))
+    } else {
+        xtab <- xtab / sum(as.double(wts)[!is.na(x)])
+    }
   }
 
   dropped <- which(xtab < threshold)
   orig <- levels(x)
 
-  if (length(dropped) > 0)
+  if (length(dropped) > 0) {
     keepers <- names(xtab[-dropped])
-  else
+  } else {
     keepers <- orig
+  }
 
-  if (length(keepers) == 0)
+  if (length(keepers) == 0) {
     keepers <- names(xtab)[which.max(xtab)]
+  }
 
-  if (other %in% keepers)
+  if (other %in% keepers) {
     rlang::abort(
-        paste0(
+      paste0(
         "The level ",
         other,
         " is already a factor level that will be retained. ",
         "Please choose a different value."
-      )
+      ),
+      call = call
     )
+  }
 
-  list(keep = orig[orig %in% keepers],
-       collapse = length(dropped) > 0,
-       other = other)
+  list(
+    keep = orig[orig %in% keepers],
+    collapse = length(dropped) > 0,
+    other = other
+  )
 }
 
 
@@ -250,19 +268,22 @@ tidy.step_other <- function(x, ...) {
   if (is_trained(x)) {
     values <- purrr::map(x$objects, function(x) x$keep)
     n <- vapply(values, length, integer(1))
-    res <- tibble(terms = rep(names(n), n),
-                  retained = unname(unlist(values)))
+    values <- vctrs::list_unchop(values, ptype = character(), name_spec = rlang::zap())
+    res <- tibble(
+      terms = rep(names(n), n),
+      retained = values
+    )
   } else {
     term_names <- sel2char(x$terms)
-    res <- tibble(terms = term_names,
-                  retained = rep(na_chr, length(term_names)))
+    res <- tibble(
+      terms = term_names,
+      retained = rep(na_chr, length(term_names))
+    )
   }
   res$id <- x$id
   res
 }
 
-
-#' @rdname tunable.recipe
 #' @export
 tunable.step_other <- function(x, ...) {
   tibble::tibble(
@@ -275,4 +296,3 @@ tunable.step_other <- function(x, ...) {
     component_id = x$id
   )
 }
-
