@@ -5,40 +5,42 @@
 #'  range of values.
 #'
 #' @inheritParams step_center
-#' @param ... One or more selector functions to choose which
-#'  variables will be scaled. See [selections()] for more
-#'  details. For the `tidy` method, these are not currently
-#'  used.
-#' @param role Not used by this step since no new variables are
-#'  created.
 #' @param min A single numeric value for the smallest value in the
 #'  range.
 #' @param max A single numeric value for the largest value in the
 #'  range.
+#' @param clipping A single logical value for determining whether
+#'  application of transformation onto new data should be forced
+#'  to be inside `min` and `max`. Defaults to TRUE.
 #' @param ranges A character vector of variables that will be
 #'  normalized. Note that this is ignored until the values are
-#'  determined by [prep.recipe()]. Setting this value will
+#'  determined by [prep()]. Setting this value will
 #'  be ineffective.
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` (the
-#'  selectors or variables selected), `min`, and `max`.
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept normalization_methods
+#' @template step-return
+#' @family normalization steps
 #' @export
 #' @details When a new data point is outside of the ranges seen in
 #'  the training set, the new values are truncated at `min` or
 #'  `max`.
-#' @examples
-#' library(modeldata)
-#' data(biomass)
 #'
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#'  # Tidying
 #'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-#'               data = biomass_tr)
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#'  `terms` (the selectors or variables selected), `min`, and `max` is
+#'  returned.
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(biomass, package = "modeldata")
+#'
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
+#'
+#' rec <- recipe(
+#'   HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
+#'   data = biomass_tr
+#' )
 #'
 #' ranged_trans <- rec %>%
 #'   step_range(carbon, hydrogen)
@@ -52,7 +54,6 @@
 #'
 #' tidy(ranged_trans, number = 1)
 #' tidy(ranged_obj, number = 1)
-
 step_range <-
   function(recipe,
            ...,
@@ -60,17 +61,19 @@ step_range <-
            trained = FALSE,
            min = 0,
            max = 1,
+           clipping = TRUE,
            ranges = NULL,
            skip = FALSE,
            id = rand_id("range")) {
     add_step(
       recipe,
       step_range_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         min = min,
         max = max,
+        clipping = clipping,
         ranges = ranges,
         skip = skip,
         id = id
@@ -79,7 +82,7 @@ step_range <-
   }
 
 step_range_new <-
-  function(terms, role, trained, min, max, ranges, skip, id) {
+  function(terms, role, trained, min, max, clipping, ranges, skip, id) {
     step(
       subclass = "range",
       terms = terms,
@@ -87,6 +90,7 @@ step_range_new <-
       trained = trained,
       min = min,
       max = max,
+      clipping = clipping,
       ranges = ranges,
       skip = skip,
       id = id
@@ -95,8 +99,8 @@ step_range_new <-
 
 #' @export
 prep.step_range <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
-  check_type(training[, col_names])
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("double", "integer"))
 
   mins <-
     vapply(training[, col_names], min, c(min = 0), na.rm = TRUE)
@@ -108,6 +112,7 @@ prep.step_range <- function(x, training, info = NULL, ...) {
     trained = TRUE,
     min = x$min,
     max = x$max,
+    clipping = x$clipping,
     ranges = rbind(mins, maxs),
     skip = x$skip,
     id = x$id
@@ -116,40 +121,46 @@ prep.step_range <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_range <- function(object, new_data, ...) {
-  tmp <- as.matrix(new_data[, colnames(object$ranges)])
-  tmp <- sweep(tmp, 2, object$ranges[1, ], "-")
-  tmp <- tmp * (object$max - object$min)
-  tmp <- sweep(tmp, 2, object$ranges[2, ] - object$ranges[1, ], "/")
-  tmp <- tmp + object$min
+  check_new_data(colnames(object$ranges), object, new_data)
 
-  tmp[tmp < object$min] <- object$min
-  tmp[tmp > object$max] <- object$max
+  for (column in colnames(object$ranges)) {
+    min <- object$ranges["mins", column]
+    max <- object$ranges["maxs", column]
 
-  tmp <- tibble::as_tibble(tmp)
-  new_data[, colnames(object$ranges)] <- tmp
-  as_tibble(new_data)
+    new_data[[column]] <- (new_data[[column]] - min) *
+      (object$max - object$min) / (max - min) + object$min
+
+    if (object$clipping) {
+      new_data[[column]] <- pmax(new_data[[column]], object$min)
+      new_data[[column]] <- pmin(new_data[[column]], object$max)
+    }
+  }
+  new_data
 }
 
 print.step_range <-
   function(x, width = max(20, options()$width - 30), ...) {
-    cat("Range scaling to [", x$min, ",", x$max, "] for ", sep = "")
-    printer(colnames(x$ranges), x$terms, x$trained, width = width)
+    title <- glue::glue("Range scaling to [{x$min},{x$max}] for ")
+    print_step(colnames(x$ranges), x$terms, x$trained, title, width)
     invisible(x)
   }
 
-#' @rdname step_range
-#' @param x A `step_range` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_range <- function(x, ...) {
   if (is_trained(x)) {
-    res <- tibble(terms = colnames(x$ranges),
-                  min = x$ranges["mins",],
-                  max = x$ranges["maxs",])
+    res <- tibble(
+      terms = colnames(x$ranges) %||% character(),
+      min = unname(x$ranges["mins", ]),
+      max = unname(x$ranges["maxs", ])
+    )
   } else {
     term_names <- sel2char(x$terms)
-    res <- tibble(terms = term_names,
-                  min = na_dbl,
-                  max = na_dbl)
+    res <- tibble(
+      terms = term_names,
+      min = na_dbl,
+      max = na_dbl
+    )
   }
   res$id <- x$id
   res
