@@ -21,25 +21,32 @@
 #' @param naming A function that defines the naming convention for
 #'  new ratio columns.
 #' @param columns The column names used in the ratios. This
-#'  argument is not populated until [prep.recipe()] is
+#'  argument is not populated until [prep()] is
 #'  executed.
 #' @template step-return
-#' @details When you [`tidy()`] this step, a tibble with columns `terms` (the
-#'  selectors or variables selected) and `denom` is returned.
+#' @details
+#'
+#' # Tidying
+#'
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#' `terms` (the selectors or variables selected) and `denom` is returned.
+#'
+#' @template case-weights-not-supported
+#'
 #' @family multivariate transformation steps
 #' @export
-#' @examples
+#' @examplesIf rlang::is_installed("modeldata")
 #' library(recipes)
-#' library(modeldata)
-#' data(biomass)
+#' data(biomass, package = "modeldata")
 #'
 #' biomass$total <- apply(biomass[, 3:7], 1, sum)
-#' biomass_tr <- biomass[biomass$dataset == "Training",]
-#' biomass_te <- biomass[biomass$dataset == "Testing",]
+#' biomass_tr <- biomass[biomass$dataset == "Training", ]
+#' biomass_te <- biomass[biomass$dataset == "Testing", ]
 #'
 #' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen +
-#'                     sulfur + total,
-#'               data = biomass_tr)
+#'   sulfur + total,
+#' data = biomass_tr
+#' )
 #'
 #' ratio_recipe <- rec %>%
 #'   # all predictors over total
@@ -51,30 +58,31 @@
 #'
 #' ratio_data <- bake(ratio_recipe, biomass_te)
 #' ratio_data
-
 step_ratio <-
   function(recipe,
            ...,
            role = "predictor",
            trained = FALSE,
            denom = denom_vars(),
-           naming = function(numer, denom)
-             make.names(paste(numer, denom, sep = "_o_")),
+           naming = function(numer, denom) {
+             make.names(paste(numer, denom, sep = "_o_"))
+           },
            columns = NULL,
            keep_original_cols = TRUE,
            skip = FALSE,
            id = rand_id("ratio")) {
-    if (is_empty(denom))
+    if (is_empty(denom)) {
       rlang::abort(
         paste0(
           "Please supply at least one denominator variable specification. ",
           "See ?selections."
         )
       )
+    }
     add_step(
       recipe,
       step_ratio_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         denom = denom,
@@ -112,14 +120,13 @@ prep.step_ratio <- function(x, training, info = NULL, ...) {
     bottom = recipes_eval_select(x$denom, training, info),
     stringsAsFactors = FALSE
   )
+  col_names <- tibble::as_tibble(col_names)
   col_names <- col_names[!(col_names$top == col_names$bottom), ]
 
-  if (nrow(col_names) == 0)
-    rlang::abort("No variables were selected for making ratios")
-  if (any(info$type[info$variable %in% col_names$top] != "numeric"))
-    rlang::abort("The ratio variables should be numeric")
-  if (any(info$type[info$variable %in% col_names$bottom] != "numeric"))
-    rlang::abort("The ratio variables should be numeric")
+  check_type(
+    training[, c(col_names$top, col_names$bottom)],
+    types = c("double", "integer")
+  )
 
   step_ratio_new(
     terms = x$terms,
@@ -136,13 +143,21 @@ prep.step_ratio <- function(x, training, info = NULL, ...) {
 
 #' @export
 bake.step_ratio <- function(object, new_data, ...) {
-  res <- new_data[, object$columns$top] /
-    new_data[, object$columns$bottom]
-  colnames(res) <-
-    apply(object$columns, 1, function(x)
-      object$naming(x[1], x[2]))
-  if (!is_tibble(res))
-    res <- as_tibble(res)
+  check_new_data(unname(object$columns$top), object, new_data)
+
+  res <- purrr::map2(
+    new_data[, object$columns$top],
+    new_data[, object$columns$bottom],
+    `/`
+  )
+
+  names(res) <- apply(
+    object$columns,
+    MARGIN = 1,
+    function(x) object$naming(x[1], x[2])
+  )
+
+  res <- tibble::new_tibble(res, nrow = nrow(new_data))
 
   keep_original_cols <- get_keep_original_cols(object)
   new_data <- bind_cols(new_data, res)
@@ -151,24 +166,14 @@ bake.step_ratio <- function(object, new_data, ...) {
     union_cols <- union(object$columns$top, object$columns$bottom)
     new_data <- new_data[, !(colnames(new_data) %in% union_cols), drop = FALSE]
   }
-
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 print.step_ratio <-
   function(x, width = max(20, options()$width - 30), ...) {
-    cat("Ratios from ")
-    if (x$trained) {
-      vars <- c(unique(x$columns$top), unique(x$columns$bottom))
-      cat(format_ch_vec(vars, width = width))
-    } else
-      cat(format_selectors(c(x$terms, x$denom), width = width))
-    if (x$trained)
-      cat(" [trained]\n")
-    else
-      cat("\n")
+    title <- "Ratios from "
+    vars <- c(unique(x$columns$top), unique(x$columns$bottom))
+    print_step(vars, c(x$terms, x$denom), x$trained, title, width)
     invisible(x)
   }
 
@@ -180,14 +185,17 @@ denom_vars <- function(...) quos(...)
 #' @export
 tidy.step_ratio <- function(x, ...) {
   if (is_trained(x)) {
-    res <- tibble(terms = unname(x$columns$top),
-                  denom = unname(x$columns$bottom))
+    res <- tibble(
+      terms = unname(x$columns$top),
+      denom = unname(x$columns$bottom)
+    )
   } else {
-    res <- tidyr::crossing(terms = sel2char(x$terms),
-                           denom = sel2char(x$denom))
+    res <- tidyr::crossing(
+      terms = sel2char(x$terms),
+      denom = sel2char(x$denom)
+    )
     res <- as_tibble(res)
   }
   res$id <- x$id
   arrange(res, terms, denom)
 }
-
