@@ -6,20 +6,28 @@
 #' @inheritParams step_center
 #' @param removals A character string that contains the names of
 #'  columns that should be removed. These values are not determined
-#'  until [prep.recipe()] is called.
+#'  until [prep()] is called.
 #' @template step-return
-#' @details When you [`tidy()`] this step, a tibble with column `terms` (the
-#'  columns that will be removed) is returned.
+#' @template filter-steps
+#' @details
+#'
+#' # Tidying
+#'
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with column
+#' `terms` (the columns that will be removed) is returned.
+#'
+#' @template case-weights-not-supported
+#'
 #' @family variable filter steps
 #' @export
-#' @examples
-#' library(modeldata)
-#' data(biomass)
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(biomass, package = "modeldata")
 #'
 #' biomass_tr <- biomass[biomass$dataset == "Training", ]
 #' biomass_te <- biomass[biomass$dataset == "Testing", ]
 #'
-#' rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
+#' rec <- recipe(
+#'   HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
 #'   data = biomass_tr
 #' )
 #'
@@ -43,7 +51,7 @@ step_rm <- function(recipe,
   add_step(
     recipe,
     step_rm_new(
-      terms = ellipse_check(...),
+      terms = enquos(...),
       role = role,
       trained = trained,
       removals = removals,
@@ -84,27 +92,13 @@ bake.step_rm <- function(object, new_data, ...) {
   if (length(object$removals) > 0) {
     new_data <- new_data[, !(colnames(new_data) %in% object$removals)]
   }
-  as_tibble(new_data)
+  new_data
 }
 
 print.step_rm <-
   function(x, width = max(20, options()$width - 22), ...) {
-    if (x$trained) {
-      if (length(x$removals) > 0) {
-        cat("Variables removed ")
-        cat(format_ch_vec(x$removals, width = width))
-      } else {
-        cat("No variables were removed")
-      }
-    } else {
-      cat("Delete terms ", sep = "")
-      cat(format_selectors(x$terms, width = width))
-    }
-    if (x$trained) {
-      cat(" [trained]\n")
-    } else {
-      cat("\n")
-    }
+    title <- "Variables removed "
+    print_step(x$removals, x$terms, x$trained, title, width)
     invisible(x)
   }
 
