@@ -69,7 +69,7 @@ step_sqrt_new <-
 #' @export
 prep.step_sqrt <- function(x, training, info = NULL, ...) {
   col_names <- recipes_eval_select(x$terms, training, info)
-  check_type(training[, col_names])
+  check_type(training[, col_names], types = c("double", "integer"))
 
   step_sqrt_new(
     terms = x$terms,
