@@ -8,19 +8,20 @@
 #' @param ... All arguments to the operator that should be returned.
 #' @param .prefix Prefix to the subclass created.
 #' @keywords internal
-#' @concept preprocessing
 #' @return An updated step or check with the new class.
 #' @export
 step <- function(subclass, ..., .prefix = "step_") {
   structure(list(...),
-            class = c(paste0(.prefix, subclass), "step"))
+    class = c(paste0(.prefix, subclass), "step")
+  )
 }
 
 #' @rdname step
 #' @export
 check <- function(subclass, ..., .prefix = "check_") {
   structure(list(...),
-            class = c(paste0(.prefix, subclass), "check"))
+    class = c(paste0(.prefix, subclass), "check")
+  )
 }
 
 #' Add a New Operation to the Current Recipe
@@ -30,8 +31,6 @@ check <- function(subclass, ..., .prefix = "check_") {
 #'
 #' @param rec A [recipe()].
 #' @param object A step or check object.
-#' @keywords datagen
-#' @concept preprocessing
 #' @return A updated [recipe()] with the new operation in the last slot.
 #' @export
 add_step <- function(rec, object) {
@@ -46,4 +45,3 @@ add_check <- function(rec, object) {
   rec$steps[[length(rec$steps) + 1]] <- object
   rec
 }
-
