@@ -1,58 +1,79 @@
 #' Convert Strings to Factors
 #'
+#' @description
 #' `step_string2factor` will convert one or more character
 #'  vectors to factors (ordered or unordered).
 #'
+#'  _Use this step only in special cases_ (see Details) and instead convert
+#'  strings to factors before using any tidymodels functions.
+#'
 #' @inheritParams step_center
-#' @inherit step_center return
-#' @param ... One or more selector functions to choose which
-#'  variables will be converted to factors. See
-#'  [selections()] for more details. For the `tidy`
-#'  method, these are not currently used.
-
-#' @param role Not used by this step since no new variables are
-#'  created.
 #' @param levels An options specification of the levels to be used
 #'  for the new factor. If left `NULL`, the sorted unique
 #'  values present when `bake` is called will be used.
 #' @param ordered A single logical value; should the factor(s) be
 #'  ordered?
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` (the
-#'  selectors or variables selected) and `ordered`.
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept variable_encodings
-#' @concept factors
+#' @template step-return
+#' @family dummy variable and encoding steps
 #' @export
-#' @details If `levels` is given, `step_string2factor` will
+#' @details
+#'
+#'  ## When should you use this step?
+#'
+#'  In most cases, if you are planning to use `step_string2factor()`
+#'  without setting `levels`, you will be better off converting
+#'  those character variables to factor variables **before using a recipe**.
+#'
+#'  This can be done using \pkg{dplyr} with the following code
+#'
+#'  ```r
+#'  df <- mutate(df, across(where(is.character), as.factor))
+#'  ```
+#'
+#'  During resampling, the complete set of values might
+#'  not be in the character data. Converting them to factors with
+#'  `step_string2factor()`  then will misconfigure the levels.
+#'
+#'  If the `levels` argument to `step_string2factor()`is used, it will
 #'  convert all variables affected by this step to have the same
-#'  levels.
+#'  levels. Because of this, you will need to know the full set of level
+#'  when you define the recipe.
 #'
-#'  Also, note that `prep` has an option `strings_as_factors` that
+#'  Also, note that [prep()] has an option `strings_as_factors` that
 #'  defaults to `TRUE`. This should be changed so that raw character
-#'  data will be applied to `step_string2factor`. However, this step
+#'  data will be applied to `step_string2factor()`. However, this step
 #'  can also take existing factors (but will leave them as-is).
-#' @seealso [step_factor2string()] [step_dummy()] [step_other()]
-#'  [step_novel()]
-#' @examples
-#' library(modeldata)
-#' data(okc)
 #'
-#' rec <- recipe(~ diet + location, data = okc)
+#'  # Tidying
+#'
+#'  When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#'  `terms` (the selectors or variables selected) and `ordered` is
+#'  returned.
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed("modeldata")
+#' data(Sacramento, package = "modeldata")
+#'
+#' # convert factor to string to demonstrate
+#' Sacramento$city <- as.character(Sacramento$city)
+#'
+#' rec <- recipe(~ city + zip, data = Sacramento)
 #'
 #' make_factor <- rec %>%
-#'   step_string2factor(diet)
-#' make_factor <- prep(make_factor,
-#'                     training = okc,
-#'                     strings_as_factors = FALSE)
+#'   step_string2factor(city)
 #'
-#' # note that `diet` is a factor
-#' bake(make_factor, new_data = NULL) %>% head
-#' okc %>% head
-#' tidy(make_factor, number = 1)
-
+#' make_factor <- prep(make_factor,
+#'   training = Sacramento
+#' )
+#'
+#' make_factor
+#'
+#' # note that `city` is a factor in recipe output
+#' bake(make_factor, new_data = NULL) %>% head()
+#'
+#' # ...but remains a string in the data
+#' Sacramento %>% head()
 step_string2factor <-
   function(recipe,
            ...,
@@ -74,7 +95,7 @@ step_string2factor <-
     add_step(
       recipe,
       step_string2factor_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         role = role,
         trained = trained,
         levels = levels,
@@ -99,30 +120,20 @@ step_string2factor_new <-
     )
   }
 
-get_ord_lvls <- function(x)
+get_ord_lvls <- function(x) {
   sort(unique(x))
+}
 
 #' @export
 prep.step_string2factor <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
-  str_check <-
-    vapply(
-      training[, col_names],
-      function(x) is.character(x) | is.factor(x),
-      logical(1)
-    )
-  if (any(!str_check))
-    rlang::abort(
-      paste0(
-        "The following variables are not character vectors: ",
-        paste0("`", names(str_check)[!str_check], "`", collapse = ", ")
-      )
-    )
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("string", "factor", "ordered"))
 
   if (is.null(x$levels)) {
     res <- lapply(training[, col_names], get_ord_lvls)
-  } else
+  } else {
     res <- x$levels
+  }
 
   ord <- rep(x$ordered, length(col_names))
   names(ord) <- col_names
@@ -139,8 +150,9 @@ prep.step_string2factor <- function(x, training, info = NULL, ...) {
 }
 
 make_factor <- function(x, lvl, ord) {
-  if (is.factor(x))
+  if (is.factor(x)) {
     return(x)
+  }
   factor(x, levels = lvl, ordered = ord)
 }
 
@@ -150,45 +162,46 @@ bake.step_string2factor <- function(object, new_data, ...) {
 
   if (is.list(object$levels)) {
     new_data[, col_names] <-
-      map2_df(new_data[, col_names],
-              object$levels,
-              make_factor,
-              ord = object$ordered[1])
+      purrr::map2(new_data[, col_names],
+        object$levels,
+        make_factor,
+        ord = object$ordered[1]
+      )
   } else {
     new_data[, col_names] <-
-      map_df(new_data[, col_names],
-             make_factor,
-             lvl = object$levels,
-             ord = object$ordered[1])
+      map(new_data[, col_names],
+        make_factor,
+        lvl = object$levels,
+        ord = object$ordered[1]
+      )
   }
-
-  if (!is_tibble(new_data))
-    new_data <- as_tibble(new_data)
   new_data
 }
 
 print.step_string2factor <-
   function(x, width = max(20, options()$width - 30), ...) {
-    cat("Factor variables from ")
-    printer(names(x$ordered), x$terms, x$trained, width = width)
+    title <- "Factor variables from "
+    print_step(names(x$ordered), x$terms, x$trained, title, width)
     invisible(x)
   }
 
 
-#' @rdname step_string2factor
-#' @param x A `step_string2factor` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_string2factor <- function(x, ...) {
   term_names <- sel2char(x$terms)
   p <- length(term_names)
   if (is_trained(x)) {
-    res <- tibble(terms = term_names,
-                  ordered = rep(x$ordered, p))
+    res <- tibble(
+      terms = term_names,
+      ordered = rep(unname(x$ordered), p)
+    )
   } else {
-    res <- tibble(terms = term_names,
-                  ordered = rep(x$ordered, p))
+    res <- tibble(
+      terms = term_names,
+      ordered = rep(unname(x$ordered), p)
+    )
   }
   res$id <- x$id
   res
 }
-
