@@ -5,10 +5,6 @@
 #'  functions that compute statistics across moving windows.
 #'
 #' @inheritParams step_center
-#' @param ... One or more selector functions to choose which
-#'  variables are affected by the step. See [selections()]
-#'  for more details.  For the `tidy` method, these are not
-#'  currently used.
 #' @param role For model terms created by this step, what analysis
 #'  role should they be assigned? If `names` is left to be
 #'  `NULL`, the rolling statistics replace the original columns
@@ -25,20 +21,13 @@
 #'  `'var'`
 #' @param columns A character string that contains the names of
 #'  columns that should be processed. These values are not
-#'  determined until [prep.recipe()] is called.
+#'  determined until [prep()] is called.
 #' @param names An optional character string that is the same
 #'  length of the number of terms selected by `terms`. If you
 #'  are not sure what columns will be selected, use the
 #'  `summary` function (see the example below). These will be
 #'  the names of the new columns created by the step.
-#' @return An updated version of `recipe` with the new step
-#'  added to the sequence of existing steps (if any). For the
-#'  `tidy` method, a tibble with columns `terms` (the
-#'  selectors or variables selected) and `statistic` (the
-#'  summary function name), and `size`.
-#' @keywords datagen
-#' @concept preprocessing
-#' @concept moving_windows
+#' @template step-return
 #' @export
 #' @details The calculations use a somewhat atypical method for
 #'  handling the beginning and end parts of the rolling statistics.
@@ -50,9 +39,18 @@
 #'  values are estimated by `median(x[1:5])` and the fourth
 #'  uses `median(x[2:6])`.
 #'
-# This step requires the \pkg{RcppRoll} package. If not installed, the
+#  This step requires the \pkg{RcppRoll} package. If not installed, the
 #'  step will stop with a note about installing the package.
-#' @examples
+#'
+#' # Tidying
+#'
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#' `terms` (the selectors or variables selected), `statistic` (the
+#' summary function name), and `size` is returned.
+#'
+#' @template case-weights-not-supported
+#'
+#' @examplesIf rlang::is_installed(c("RcppML", "ggplot2"))
 #' library(recipes)
 #' library(dplyr)
 #' library(rlang)
@@ -67,18 +65,17 @@
 #' sim_dat$x3 <- rnorm(n)
 #'
 #' rec <- recipe(y1 + y2 ~ x1 + x2 + x3, data = sim_dat) %>%
-#'   step_window(starts_with("y"), size = 7, statistic = "median",
-#'               names = paste0("med_7pt_", 1:2),
-#'               role = "outcome") %>%
 #'   step_window(starts_with("y"),
-#'               names = paste0("mean_3pt_", 1:2),
-#'               role = "outcome")
+#'     size = 7, statistic = "median",
+#'     names = paste0("med_7pt_", 1:2),
+#'     role = "outcome"
+#'   ) %>%
+#'   step_window(starts_with("y"),
+#'     names = paste0("mean_3pt_", 1:2),
+#'     role = "outcome"
+#'   )
 #' rec <- prep(rec, training = sim_dat)
 #'
-#' # If you aren't sure how to set the names, see which variables are selected
-#' # and the order that they are selected:
-#' terms_select(info = summary(rec), terms = quos(starts_with("y")))
-#'
 #' smoothed_dat <- bake(rec, sim_dat, everything())
 #'
 #' ggplot(data = sim_dat, aes(x = x1, y = y1)) +
@@ -100,8 +97,6 @@
 #' ggplot(smoothed_dat, aes(x = original, y = y1)) +
 #'   geom_point() +
 #'   theme_bw()
-
-
 step_window <-
   function(recipe,
            ...,
@@ -114,13 +109,15 @@ step_window <-
            names = NULL,
            skip = FALSE,
            id = rand_id("window")) {
-    if (!(statistic %in% roll_funs) | length(statistic) != 1)
+    if (!is_call(statistic) &&
+      (!(statistic %in% roll_funs) | length(statistic) != 1)) {
       rlang::abort(
         paste0(
-        "`statistic` should be one of: ",
-        paste0("'", roll_funs, "'", collapse = ", ")
-          )
+          "`statistic` should be one of: ",
+          paste0("'", roll_funs, "'", collapse = ", ")
         )
+      )
+    }
 
     ## ensure size is odd, integer, and not too small
     if (!is_tune(size) & !is_varying(size)) {
@@ -131,17 +128,18 @@ step_window <-
       if (!is.integer(size)) {
         tmp <- size
         size <- as.integer(size)
-        if (!isTRUE(all.equal(tmp, size)))
+        if (!isTRUE(all.equal(tmp, size))) {
           rlang::warn(
             paste0(
-            "`size` was not an integer (",
-            tmp,
-            ") and was ",
-            "converted to ",
-            size,
-            "."
+              "`size` was not an integer (",
+              tmp,
+              ") and was ",
+              "converted to ",
+              size,
+              "."
             )
           )
+        }
       }
       if (size %% 2 == 0) {
         rlang::abort("`size` should be odd.")
@@ -153,7 +151,7 @@ step_window <-
     add_step(
       recipe,
       step_window_new(
-        terms = ellipse_check(...),
+        terms = enquos(...),
         trained = trained,
         role = role,
         size = size,
@@ -188,19 +186,19 @@ step_window_new <-
 
 #' @export
 prep.step_window <- function(x, training, info = NULL, ...) {
-  col_names <- eval_select_recipes(x$terms, training, info)
-
-  if (any(info$type[info$variable %in% col_names] != "numeric"))
-    rlang::abort("The selected variables should be numeric")
+  col_names <- recipes_eval_select(x$terms, training, info)
+  check_type(training[, col_names], types = c("double", "integer"))
 
   if (!is.null(x$names)) {
-    if (length(x$names) != length(col_names))
+    if (length(x$names) != length(col_names)) {
       rlang::abort(
-        paste0("There were ", length(col_names), " term(s) selected but ",
-           length(x$names), " values for the new features ",
-           "were passed to `names`."
+        paste0(
+          "There were ", length(col_names), " term(s) selected but ",
+          length(x$names), " values for the new features ",
+          "were passed to `names`."
         )
       )
+    }
   }
 
   step_window_new(
@@ -223,8 +221,9 @@ roller <- function(x, stat = "mean", window = 3L, na_rm = TRUE) {
   m <- length(x)
 
   gap <- floor(window / 2)
-  if (m - window <= 2)
+  if (m - window <= 2) {
     rlang::abort("The window is too large.")
+  }
 
   ## stats for centered window
   opts <- list(
@@ -247,19 +246,25 @@ roller <- function(x, stat = "mean", window = 3L, na_rm = TRUE) {
 
 #' @export
 bake.step_window <- function(object, new_data, ...) {
+  check_new_data(names(object$columns), object, new_data)
+
   for (i in seq(along.with = object$columns)) {
     if (!is.null(object$names)) {
       new_data[, object$names[i]] <-
-        roller(x = getElement(new_data, object$columns[i]),
-               stat = object$statistic,
-               na_rm = object$na_rm,
-               window = object$size)
+        roller(
+          x = getElement(new_data, object$columns[i]),
+          stat = object$statistic,
+          na_rm = object$na_rm,
+          window = object$size
+        )
     } else {
       new_data[, object$columns[i]] <-
-        roller(x = getElement(new_data, object$columns[i]),
-               stat = object$statistic,
-               na_rm = object$na_rm,
-               window = object$size)
+        roller(
+          x = getElement(new_data, object$columns[i]),
+          stat = object$statistic,
+          na_rm = object$na_rm,
+          window = object$size
+        )
     }
   }
   new_data
@@ -268,20 +273,12 @@ bake.step_window <- function(object, new_data, ...) {
 
 print.step_window <-
   function(x, width = max(20, options()$width - 28), ...) {
-    cat("Moving ", x$size, "-point ", x$statistic, " on ", sep = "")
-    if (x$trained) {
-      cat(format_ch_vec(x$columns, width = width))
-    } else
-      cat(format_selectors(x$terms, width = width))
-    if (x$trained)
-      cat(" [trained]\n")
-    else
-      cat("\n")
+    title <- glue::glue("Moving {x$size}-point {x$statistic} on ")
+    print_step(x$columns, x$terms, x$trained, title, width)
     invisible(x)
   }
 
-#' @rdname step_window
-#' @param x A `step_window` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.step_window <- function(x, ...) {
   out <- simple_terms(x, ...)
@@ -291,15 +288,13 @@ tidy.step_window <- function(x, ...) {
   out
 }
 
-
-#' @rdname tunable.step
 #' @export
 tunable.step_window <- function(x, ...) {
   tibble::tibble(
     name = c("statistic", "window"),
     call_info = list(
       list(pkg = "dials", fun = "summary_stat"),
-      list(pkg = "dials", fun = "window")
+      list(pkg = "dials", fun = "window_size")
     ),
     source = "recipe",
     component = "step_window",
@@ -308,7 +303,7 @@ tunable.step_window <- function(x, ...) {
 }
 
 
-#' @rdname required_pkgs.step
+#' @rdname required_pkgs.recipe
 #' @export
 required_pkgs.step_window <- function(x, ...) {
   c("RcppRoll")
