@@ -55,8 +55,8 @@ create_documentation <- function(name,
 #'  preprocessing have been estimated.
 #' <additional args here>
 #' @param skip A logical. Should the step be skipped when the
-#'  recipe is baked by [bake.recipe()]? While all operations are baked
-#'  when [prep.recipe()] is run, some operations may not be able to be
+#'  recipe is baked by [bake()]? While all operations are baked
+#'  when [prep()] is run, some operations may not be able to be
 #'  conducted on new data (e.g. processing the outcome variable(s)).
 #'  Care should be taken when using `skip = TRUE` as it may affect
 #'  the computations for subsequent operations
@@ -66,6 +66,11 @@ create_documentation <- function(name,
 #' @export
 #' @details <describe details>
 #'
+#' # Tidying
+#'
+#' When you [`tidy()`][tidy.recipe()] this step, a tibble with columns
+#' <describe tidying> is returned.
+#'
 #' @examples
 
   ")
@@ -84,7 +89,7 @@ create_function <- function(name, which) {
       add_{which}(
         recipe,
         {which}_{name}_new(
-          terms = ellipse_check(...),
+          terms = enquos(...),
           trained = trained,
           role = role,
           <additional args here>
@@ -150,8 +155,8 @@ create_print_method <- function(name, which) {
   glue('
 print.{which}_{name} <-
   function(x, width = max(20, options()$width - 30), ...) {{
-    cat("<describe action here> ", sep = "")
-    printer(names(x$means), x$terms, x$trained, width = width)
+    title <- "<describe action here> "
+    print_step(names(x$means), x$terms, x$trained, title, width)
     invisible(x)
   }}
 
@@ -160,8 +165,7 @@ print.{which}_{name} <-
 
 create_tidy_method <- function(name, which) {
   glue("
-#' @rdname {which}_{name}
-#' @param x A `{which}_{name}` object.
+#' @rdname tidy.recipe
 #' @export
 tidy.{which}_{name} <- function(x, ...) {{
   if (is_trained(x)) {{
