@@ -1,17 +1,15 @@
 library(testthat)
-library(kernlab)
 library(recipes)
 
 set.seed(131)
-tr_dat <- matrix(rnorm(100*6), ncol = 6)
-te_dat <- matrix(rnorm(20*6), ncol = 6)
+tr_dat <- matrix(rnorm(100 * 6), ncol = 6)
+te_dat <- matrix(rnorm(20 * 6), ncol = 6)
 colnames(tr_dat) <- paste0("X", 1:6)
 colnames(te_dat) <- paste0("X", 1:6)
 
 rec <- recipe(X1 ~ ., data = tr_dat)
 
-test_that('correct kernel PCA values', {
-  skip_if_not_installed("dimRed")
+test_that("correct kernel PCA values", {
   skip_if_not_installed("kernlab")
 
   kpca_rec <- rec %>%
@@ -22,9 +20,10 @@ test_that('correct kernel PCA values', {
   pca_pred <- bake(kpca_trained, new_data = te_dat, all_predictors())
   pca_pred <- as.matrix(pca_pred)
 
-  pca_exp <- kpca(as.matrix(tr_dat[, -1]),
-                  kernel = "rbfdot",
-                  kpar = list(sigma = 0.2))
+  pca_exp <- kernlab::kpca(as.matrix(tr_dat[, -1]),
+    kernel = "rbfdot",
+    kpar = list(sigma = 0.2)
+  )
 
   pca_pred_exp <- kernlab::predict(pca_exp, te_dat[, -1])[, 1:kpca_trained$steps[[1]]$num_comp]
   colnames(pca_pred_exp) <- paste0("kPC", 1:kpca_trained$steps[[1]]$num_comp)
@@ -41,38 +40,39 @@ test_that('correct kernel PCA values', {
   expect_equal(tidy(kpca_trained, 1), kpca_tibble)
 })
 
-test_that('printing', {
-  skip_if_not_installed("dimRed")
+test_that("printing", {
+  skip_if(packageVersion("rlang") < "1.0.0")
   skip_if_not_installed("kernlab")
 
   kpca_rec <- rec %>%
     step_kpca_rbf(X2, X3, X4, X5, X6)
   expect_snapshot(kpca_rec)
-  expect_snapshot(prep(kpca_rec, training = tr_dat, verbose = TRUE))
+  expect_snapshot(prep(kpca_rec))
 })
 
 
-test_that('No kPCA comps', {
+test_that("No kPCA comps", {
   pca_extract <- rec %>%
     step_kpca_rbf(X2, X3, X4, X5, X6, num_comp = 0, id = "") %>%
     prep()
 
   expect_equal(
-    names(juice(pca_extract)),
+    names(bake(pca_extract, new_data = NULL)),
     paste0("X", c(2:6, 1))
   )
-  expect_true(inherits(pca_extract$steps[[1]]$res, "list"))
-  expect_snapshot(pca_extract)
+  expect_null(pca_extract$steps[[1]]$res)
   expect_equal(
     tidy(pca_extract, 1),
     tibble::tibble(terms = paste0("X", 2:6), id = "")
   )
+  skip_if(packageVersion("rlang") < "1.0.0")
+  expect_snapshot(pca_extract)
 })
 
 
-test_that('tunable', {
+test_that("tunable", {
   rec <-
-    recipe(~ ., data = iris) %>%
+    recipe(~., data = iris) %>%
     step_kpca_rbf(all_predictors())
   rec_param <- tunable.step_kpca_rbf(rec$steps[[1]])
   expect_equal(rec_param$name, c("num_comp", "sigma"))
@@ -81,13 +81,11 @@ test_that('tunable', {
   expect_equal(nrow(rec_param), 2)
   expect_equal(
     names(rec_param),
-    c('name', 'call_info', 'source', 'component', 'component_id')
+    c("name", "call_info", "source", "component", "component_id")
   )
 })
 
-test_that('keep_original_cols works', {
-
-  skip_if_not_installed("dimRed")
+test_that("keep_original_cols works", {
   skip_if_not_installed("kernlab")
 
   kpca_rec <- rec %>%
@@ -99,14 +97,14 @@ test_that('keep_original_cols works', {
 
   expect_equal(
     colnames(pca_pred),
-    c("X2", "X3", "X4", "X5", "X6",
-      "kPC1", "kPC2", "kPC3", "kPC4", "kPC5")
+    c(
+      "X2", "X3", "X4", "X5", "X6",
+      "kPC1", "kPC2", "kPC3", "kPC4", "kPC5"
+    )
   )
-
 })
 
-test_that('can prep recipes with no keep_original_cols', {
-  skip_if_not_installed("dimRed")
+test_that("can prep recipes with no keep_original_cols", {
   skip_if_not_installed("kernlab")
 
   kpca_rec <- rec %>%
@@ -114,8 +112,8 @@ test_that('can prep recipes with no keep_original_cols', {
 
   kpca_rec$steps[[1]]$keep_original_cols <- NULL
 
-  expect_snapshot(
-    kpca_trained <- prep(kpca_rec, training = tr_dat, verbose = FALSE),
+  suppressWarnings(
+    kpca_trained <- prep(kpca_rec, training = tr_dat, verbose = FALSE)
   )
 
   expect_error(
@@ -123,4 +121,60 @@ test_that('can prep recipes with no keep_original_cols', {
     NA
   )
 
+  skip_if(packageVersion("rlang") < "1.0.0")
+  expect_snapshot(
+    kpca_trained <- prep(kpca_rec, training = tr_dat, verbose = FALSE)
+  )
+})
+
+test_that("empty selection prep/bake is a no-op", {
+  rec1 <- recipe(mpg ~ ., mtcars)
+  rec2 <- step_kpca_rbf(rec1)
+
+  rec1 <- prep(rec1, mtcars)
+  rec2 <- prep(rec2, mtcars)
+
+  baked1 <- bake(rec1, mtcars)
+  baked2 <- bake(rec2, mtcars)
+
+  expect_identical(baked1, baked2)
+})
+
+test_that("empty selection tidy method works", {
+  rec <- recipe(mpg ~ ., mtcars)
+  rec <- step_kpca_rbf(rec)
+
+  expect <- tibble(terms = character(), id = character())
+
+  expect_identical(tidy(rec, number = 1), expect)
+
+  rec <- prep(rec, mtcars)
+
+  expect_identical(tidy(rec, number = 1), expect)
+})
+
+test_that("empty printing", {
+  skip_if(packageVersion("rlang") < "1.0.0")
+  rec <- recipe(mpg ~ ., mtcars)
+  rec <- step_kpca_rbf(rec)
+
+  expect_snapshot(rec)
+
+  rec <- prep(rec, mtcars)
+
+  expect_snapshot(rec)
+})
+
+test_that("bake method errors when needed non-standard role columns are missing", {
+  skip_if_not_installed("kernlab")
+
+  kpca_rec <- rec %>%
+    step_kpca_rbf(X2, X3, X4, X5, X6) %>%
+    update_role(X2, X3, X4, X5, X6, new_role = "potato") %>%
+    update_role_requirements(role = "potato", bake = FALSE)
+
+  kpca_trained <- prep(kpca_rec, training = tr_dat, verbose = FALSE)
+
+  expect_error(bake(kpca_trained, new_data = te_dat[, 1:3]),
+               class = "new_data_missing_column")
 })
