@@ -1,19 +1,20 @@
 library(testthat)
 library(recipes)
-library(modeldata)
 library(splines)
-data(biomass)
+skip_if_not_installed("modeldata")
+data(biomass, package = "modeldata")
 
 # ------------------------------------------------------------------------------
 
-biomass_tr <- biomass[biomass$dataset == "Training",]
-biomass_te <- biomass[biomass$dataset == "Testing",]
+biomass_tr <- biomass[biomass$dataset == "Training", ]
+biomass_te <- biomass[biomass$dataset == "Testing", ]
 
 rec <- recipe(HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
-              data = biomass_tr)
+  data = biomass_tr
+)
 # ------------------------------------------------------------------------------
 
-test_that('correct basis functions', {
+test_that("correct basis functions", {
   with_ns <- rec %>%
     step_ns(carbon, hydrogen)
 
@@ -71,16 +72,16 @@ test_that('correct basis functions', {
 })
 
 
-test_that('printing', {
-  with_ns <- rec %>%  step_ns(carbon, hydrogen)
-  expect_output(print(with_ns))
-  expect_output(prep(with_ns, training = biomass_tr, verbose = TRUE))
+test_that("printing", {
+  with_ns <- rec %>% step_ns(carbon, hydrogen)
+  expect_snapshot(print(with_ns))
+  expect_snapshot(prep(with_ns))
 })
 
 
-test_that('tunable', {
+test_that("tunable", {
   rec <-
-    recipe(~ ., data = iris) %>%
+    recipe(~., data = iris) %>%
     step_ns(all_predictors())
   rec_param <- tunable.step_ns(rec$steps[[1]])
   expect_equal(rec_param$name, c("deg_free"))
@@ -89,6 +90,56 @@ test_that('tunable', {
   expect_equal(nrow(rec_param), 1)
   expect_equal(
     names(rec_param),
-    c('name', 'call_info', 'source', 'component', 'component_id')
+    c("name", "call_info", "source", "component", "component_id")
   )
 })
+
+test_that("empty selection prep/bake is a no-op", {
+  rec1 <- recipe(mpg ~ ., mtcars)
+  rec2 <- step_ns(rec1)
+
+  rec1 <- prep(rec1, mtcars)
+  rec2 <- prep(rec2, mtcars)
+
+  baked1 <- bake(rec1, mtcars)
+  baked2 <- bake(rec2, mtcars)
+
+  expect_identical(baked1, baked2)
+})
+
+test_that("empty selection tidy method works", {
+  rec <- recipe(mpg ~ ., mtcars)
+  rec <- step_ns(rec)
+
+  expect <- tibble(terms = character(), id = character())
+
+  expect_identical(tidy(rec, number = 1), expect)
+
+  rec <- prep(rec, mtcars)
+
+  expect_identical(tidy(rec, number = 1), expect)
+})
+
+test_that("empty printing", {
+  skip_if(packageVersion("rlang") < "1.0.0")
+  rec <- recipe(mpg ~ ., mtcars)
+  rec <- step_ns(rec)
+
+  expect_snapshot(rec)
+
+  rec <- prep(rec, mtcars)
+
+  expect_snapshot(rec)
+})
+
+test_that("bake method errors when needed non-standard role columns are missing", {
+  with_ns <- rec %>%
+    step_ns(carbon, hydrogen) %>%
+    update_role(carbon, hydrogen, new_role = "potato") %>%
+    update_role_requirements(role = "potato", bake = FALSE)
+
+  with_ns <- prep(with_ns, training = biomass_tr, verbose = FALSE)
+
+  expect_error(bake(with_ns, new_data = biomass_tr[, c(-3)]),
+               class = "new_data_missing_column")
+})
