@@ -1,24 +1,31 @@
 library(testthat)
 library(recipes)
 library(dplyr)
-library(modeldata)
+skip_if_not_installed("modeldata")
 
 ## -----------------------------------------------------------------------------
 
 data(biomass, package = "modeldata")
 
-biom_tr <- biomass %>% dplyr::filter(dataset == "Training") %>% dplyr::select(-dataset, -sample)
-biom_te <- biomass %>% dplyr::filter(dataset == "Testing")  %>% dplyr::select(-dataset, -sample, -HHV)
+biom_tr <- biomass %>%
+  dplyr::filter(dataset == "Training") %>%
+  dplyr::select(-dataset, -sample)
+biom_te <- biomass %>%
+  dplyr::filter(dataset == "Testing") %>%
+  dplyr::select(-dataset, -sample, -HHV)
 
 load(test_path("test_pls_old.RData"))
 
 ## -----------------------------------------------------------------------------
 
-test_that('check old PLS scores from recipes version <= 0.1.12', {
-  new_values_tr <- juice(old_pls, all_predictors())
+test_that("check old PLS scores from recipes version <= 0.1.12", {
+  new_values_tr <- bake(old_pls, new_data = NULL, all_predictors())
   expect_equal(new_values_tr, old_pls_tr)
 
   # Capture known warning about `keep_original_cols`
-  expect_snapshot(new_values_te <- bake(old_pls, biom_te))
+
+  suppressWarnings(new_values_te <- bake(old_pls, biom_te))
   expect_equal(new_values_te, old_pls_te)
+  skip_if(packageVersion("rlang") < "1.0.0")
+  expect_snapshot(new_values_te <- bake(old_pls, biom_te))
 })
