@@ -21,7 +21,7 @@ test_that("basic usage", {
     slice(1:75) %>%
     slice(1:5)
 
-  rec_train <- juice(prepped)
+  rec_train <- bake(prepped, new_data = NULL)
   expect_equal(dplyr_train, rec_train)
 
   dplyr_test <-
@@ -47,7 +47,7 @@ test_that("skip = FALSE", {
     slice(1:75) %>%
     slice(1:5)
 
-  rec_train <- juice(prepped)
+  rec_train <- bake(prepped, new_data = NULL)
   expect_equal(dplyr_train, rec_train)
 
   dplyr_test <-
@@ -73,7 +73,7 @@ test_that("quasiquotation", {
     slice(1:75) %>%
     slice(values)
 
-  rec_1_train <- juice(prepped_1)
+  rec_1_train <- bake(prepped_1, new_data = NULL)
   expect_equal(dplyr_train, rec_1_train)
 
   expect_error(
@@ -93,21 +93,10 @@ test_that("quasiquotation", {
     prepped_2 <- prep(rec_2, training = iris %>% slice(1:75)),
     regexp = NA
   )
-  rec_2_train <- juice(prepped_2)
+  rec_2_train <- bake(prepped_2, new_data = NULL)
   expect_equal(dplyr_train, rec_2_train)
 })
 
-
-test_that("no input", {
-  no_inputs <-
-    iris_rec %>%
-    step_slice() %>%
-    prep(training = iris) %>%
-    juice(composition = "data.frame")
-  expect_equal(no_inputs, iris)
-})
-
-
 test_that("printing", {
   rec <- iris_rec %>% step_slice(1:2)
   expect_snapshot(print(rec))
